#pragma rtGlobals=1			// Use modern global access method
#include  <All IP Procedures>
#include  <Global Fit 2>
#include  <Image Saver>
#include <Image Common>
#include <Image Threshold Panel>
#include "ACL_TabUtilities"	// Simplifies making of tabs.  Located in "/Applications/Igor Pro Folder/User Procedures/ACL_tabcontrol"

Menu "FLIM"					// New commands that appear in "Macros" drop-down menu
//	"-"
	"Initialize Analysis", whiplash()
	"Recreate Panel", AnalysisPanel()
	"--------"
	"Globally Fit Histograms",GlobalFitHistogramsPanel()
	"--------"
	"Display Options", DisplayOptions()
	"PrettyLayout", PrettyLayout()
	"Create Background Image", CreateBackgroundImage()
	"Create Beta Image", CreateBetaImage()
	"--------"
	Submenu "Color Scheme"
		"Blue",SetColorScheme(0)
		"Bright",SetColorScheme(1)
		"Fuchsia",SetColorScheme(2)
		"Aqua",SetColorScheme(3)
	End
//	"-"
End

Function whiplash()				// Defines "Initialize Analysis" procedure
	Initialize()
	AnalysisPanel()
	HideProcedures
End

Function Initialize()			// Initializes global variables and tables
	// Ensures that data is not mistakenly overwritten
	if (DataFolderExists("root:Data"))
		DoAlert 1, "Are you sure you want to restart?"
		if (V_Flag == 2)	// no
			Abort
		endif
	endif
	
	// Define data and results folders
	NewDataFolder/O/S root:Data
	NewDataFolder/O root:Analysis
	NewDataFolder/O root:Analysis:Manual
	NewDataFolder/O root:Analysis:Manual:IRF	
	NewDataFolder/O root:Analysis:Manual:Single
	NewDataFolder/O root:Analysis:Manual:Double	
	NewDataFolder/O root:Analysis:Manual:Triple
	NewDataFolder/O root:Analysis:Fits
	NewDataFolder/O root:Analysis:Fits:IRF	
	NewDataFolder/O root:Analysis:Fits:Single
	NewDataFolder/O root:Analysis:Fits:Double
	NewDataFolder/O root:Analysis:Fits:Triple
	NewDataFolder/O root:Analysis:Pixels
	NewDataFolder/O root:Analysis:ROIMasks
	Variable/G MaxData=0								// Number of loaded datasets
	Variable/G Show=0	
	Variable/G Colors=3
	
	// Program Version and Authors
	String/G Version = "\f02p\f00FLIM.ipf"
	String/G Authors = "Written by K. A. Walther & A. Kinkhabwala"
	
	// Dislpay Option Flags
	Variable/G TimeHistFlag = 1
	SetDataFolder root:Analysis:Pixels
	Variable/G IntensityFlag = 1, SignalFlag = 0, BackgroundFlag = 0, BetaFlag = 0, AlphaFlag = 1, AvgTauFlag = 1
	Variable/G BetaWeightedFlag =0, AlphaWeightedFlag = 1, AvgTauWeightedFlag = 1
	Variable/G DisplayLowerTau=1, DisplayUpperTau=3, DisplayLowerAlpha=0, DisplayUpperAlpha=1
	
	// Display Options for images
	String/G ModGraph = "SetAxis/A/R left; ModifyGraph standoff=0, tick=3,noLabel=2, width=0, height={Plan,1,left,top}, margin=2, margin(right)=50"
	String/G Color1 = "ColorScale/C/N=text0/A=RC/X=-24/Y=5/Z=0/F=0 image="
	String/G Color2 = ",fsize=9"
	String/G ColorW = "ColorScale/C/N=text0/A=RC/X=-24/Y=5/Z=0/F=0 ctab="
	
// Load tab ************************************************* 
	// File Info variables
	SetDataFolder root:Data	
	Make/O/N=(100,14) FileInfo_Variables = 0
	Make/O/T labels={"xSize", "ySize", "TotalCounts", "TotalFrames","Dwelltime","Binsize","BinsUsed","SyncPeriod", "TimeperFrame", "StartFrame", "EndFrame", "Binning", "Channel", "TimeWindow"}
	SetDimLabel 0, 0, Displayed, FileInfo_Variables
	Variable i
	for (i = 0; i < 14; i += 1)
		SetDimLabel 1, i, $labels[i], FileInfo_Variables		
	endfor
	FileInfo_Variables[][%Binning] = 1		
	Make/O/T/N=(100,4) FileInfo_Strings = ""
	Make/O/T labels={"ScanPattern", "MeasurementMode", "Filetime", "CommentField"}
	SetDimLabel 0, 0, Displayed, FileInfo_Strings
	for (i = 0; i < 4; i += 1)
		SetDimLabel 1, i, $labels[i], FileInfo_Strings
	endfor
	
	// Variables for creating the histogram and analyzing the data
	Variable/G Starts=0, Ends=0
	
	// Contains the data filename that is currently displayed
	String/G Filename
	Make/O/N=4/T CurrName
	KillWaves/Z labels
	SetDataFolder root:Analysis
	Make/O/N=500	SelectedTab=0
	Variable/G percentFinished
	
// IRF, Single, Double, and Triple tabs ******************************************	
	SetDataFolder root:Analysis:Pixels
	Variable/G PreprocessFlag=1, LifetimeFlag = 0

	// Maintain parameter settings for each individual dataset
	SetDataFolder root:Analysis:Manual
	Make/O/N=(100,65) TrialValuesIRF=0		
	Make/O/T labels={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1", "tau2", "bet", "tau3", "gamm", "alpha", "avgtau_alpha", "avgtau_beta"}
	Make/O values={100000, 1, 0.03, 	    0.05,   0.25,    0.1, 		0.1,   1000,  2.6,     1.2,      0.2,     1.6, 		0.2,  0,		0,				0}
	SetDimLabel 0, 0, Displayed, TrialValuesIRF
	Variable N = numpnts(values)
	for (i = 0; i < N; i += 1)
		TrialValuesIRF[][i]= values[i]
		SetDimLabel 1, i, $labels[i], TrialValuesIRF		
	endfor
	// same for Fix...
	Duplicate/T/O labels, labels2
	labels2 = "Fix" + labels
	values ={0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
	for (i = 0; i < N; i += 1)
		TrialValuesIRF[][N+i]= values[i]
		SetDimLabel 1, N+i, $labels2[i], TrialValuesIRF		
	endfor
	// same for LL...
	labels2 = "LL" + labels
	values ={0, 0, 0.02, 0.01, 0.1, 0, 0, 0, 0, 0, 0, 0, 0}
	for (i = 0; i < N; i += 1)
		TrialValuesIRF[][2*N+i]= values[i]
		SetDimLabel 1, 2*N+i, $labels2[i], TrialValuesIRF		
	endfor
	// same for UL...
	labels2 = "UL" + labels
	values ={1e12, 2, 0.05, 0.2, 0.6, 0.5, 50, 1e8, 5, 5, 1, 5, 1}
	for (i = 0; i < N; i += 1)
		TrialValuesIRF[][3*N+i]= values[i]
		SetDimLabel 1, 3*N+i, $labels2[i], TrialValuesIRF		
	endfor
	// Add label for the time-course window
	SetDimLabel 1, 4*N, TimeWindow, TrialValuesIRF
	Duplicate/O TrialValuesIRF, TrialValuesSingle, TrialValuesDouble, TrialValuesTriple
	
	// so far so good; let's make the equivalent fit tables
	SetDataFolder root:Analysis:Fits
	Make/O/N=(100,65) FitValuesIRF=0
	SetDimLabel 0, 0, Displayed, FitValuesIRF
	for (i = 0; i < N; i += 1)
		SetDimLabel 1, i, $labels[i], FitValuesIRF		
	endfor
	SetDimLabel 1,N, Chisq , FitValuesIRF
	labels2 = labels + "Error"
	for (i = 0; i < N; i += 1)
		SetDimLabel 1, N+i+1, $labels2[i], FitValuesIRF		
	endfor
	Duplicate/O FitValuesIRF, FitValuesSingle, FitValuesDouble, FitValuesTriple
	
	// ROI variables
	SetDataFolder root:Analysis:ROIMasks
	Make/O/N=20/T ROIMaskList
	ROIMaskList[0]="No ROI"
	Make/O/N=(100,20)/T ROIMaskTable
	ROIMaskTable[0][]="No ROI"
	
	KillWaves/Z labels, values, labels2
End

//****************************** Create "FLIM Analysis Panel" *************************************//
Function AnalysisPanel()
	// Build panel
	NVAR Colors=root:Data:Colors
	if (Colors == 0)	// blue
		String back = "(5000,24817,52428)"
		String panel = "(56151,63031,65535)"
		String sub = "(49151,60031,65535)"
		String buttons = "(4768,34615,65535)"
		String line = "(65535,49151,55704)"
		String chi = "(65535,65533,32768)"
	elseif (Colors == 1)
		back = "(4*65535/256,48*65535/256,70*65535/256)"
		panel = "(182*65535/256,244*65535/256,110*65535/256)"
		sub = "(124*65535/256,231*65535/256,0*65535/256)"
		buttons = "(256*65535/256,54*65535/256,0*65535/256)"
		line = "(256*65535/256,105*65535/256,65*65535/256)"
		chi = "(100*65535/256,174*65533/256,208*65533/256)"
	elseif (Colors == 2)
		back = "(0*65535/256,0*65535/256,102*65535/256)"
		panel = "(255*65535/256,255*65535/256,204*65535/256)"
		sub = "(204*65535/256,204*65535/256,255*65535/256)"
		buttons = "(153*65535/256,0*65535/256,102*65535/256)"
		line = "(153*65535/256,0*65535/256,102*65535/256)"
		chi = "(255*65535/256,255*65535/256,204*65535/256)"
	elseif (Colors == 3)
		back = "(0*65535/256,99*65535/256,150*65535/256)"
		panel = "(198*65535/256,240*65535/256,254*65535/256)"
		sub = "(156*65535/256,226*65535/256,252*65535/256)"
		buttons = "(190*65535/256,33*65535/256,50*65535/256)"
		line = "(255*65535/256,51*65535/256,77*65535/256)"
		chi = "(256*65535/256,256*65535/256,256*65535/256)"
	endif
	Variable i, k
	String font = ",font=\"Arial\",fSize=13"
	PauseUpdate; Silent 1
	DoWindow/F/K FLIMpanel
	NewPanel/K=2/W=(0,0,625,565)  as "FLIM Analysis Panel"
	DoWindow/C FLIMpanel
	SetWindow FLIMpanel,hook(testhook)=KeyboardPanelHook 
	Execute ("ModifyPanel cbRGB=" +back)
	
	// Version and Author information
	TitleBox VersionDisplay,anchor=LT,pos={5,550},size={183,12},frame=0,variable=root:Data:Version
	Execute "TitleBox VersionDisplay,fColor=(65535,65535,65535),labelBack=" + back + font + ",fSize=9"
	TitleBox AuthorDisplay,anchor=LT,pos={435,550},size={183,12},frame=0,variable=root:Data:Authors
	Execute "TitleBox AuthorDisplay,fColor=(65535,65535,65535),labelBack=" + back + font + ",fSize=9"
	
	// Header containing the dataset information
	Execute "SetVariable setvarWave,pos={20,10},size={100,20},proc=DisplayData,value=root:Data:Show,title=\"Show\",limits={0,1000,1},fColor=(65535,65535,65535)" + font
	Execute "SetVariable setvarName, pos={140,10},size={470,20},limits={0,inf,0},title=\" \",value=root:Data:CurrName[0]"+ font
	Execute "SetVariable setvarName2, pos={140,30},size={470,20},limits={0,inf,0},title=\" \",value=root:Data:CurrName[1]"+ font
	// Make tabs
	Execute "TabControl AnalysisTabs,pos={5,55},size={615,490},fstyle=1,proc=MyTabControl,userdata(currenttab)=  \"0\",labelBack=" + panel + font
	TabControl AnalysisTabs,tabLabel(0)="Load",tabLabel(1)="IRF",tabLabel(2)="Single",tabLabel(3)="Double",tabLabel(4)="Triple",tabLabel(5)="ROI",value= 0  
	
	//******************************** Load Tab ***************************************************//
	String tabs = font + ",userdata(tabnum)=  \"0\",userdata(tabcontrol)=  \"AnalysisTabs\""
	Make/O/T file = {"Variables", "Variables", "Variables", "Variables", "Variables", "Variables", "Variables", "Variables", "Strings", "Strings", "Strings", "Strings", "Variables", "Variables"}
	Make/O/T labels = {"xSize", "ySize", "TotalCounts","TotalFrames", "Binsize", "BinsUsed", "Dwelltime", "SyncPeriod","ScanPattern", "MeasurementMode", "Filetime", "CommentField", "StartFrame", "EndFrame"}
	Make/O/T pos = {"20,150", "120,150", "220,150",     "440,150",       "20,175",  "150,175",    "280,175",     "440,175",    "20,200",        "310,200",                  "20,225",   "20,250",		"20,335",	  "180,335"}
	Make/O/T size1 = {"90", 	"90", 	     "210", 		"150",		 "120",		 "120",	    "150",			 "150",	   "280", 			"280", 				"400",	   "400",			 "150",		 "135"}
	Execute "Button buttonSelect,pos={175,90},size={180,30},proc=SelectArrivalTimeFile,title=\"Select Arrival-Time File\",fColor=" + buttons + tabs
	Execute "GroupBox groupInfo,pos={ 12,125},size={596,155},title=\"File Information\",frame=0" + tabs
	Execute "GroupBox groupHist,pos={ 12,285},size={596,105},title=\"Create Histograms\",frame=0" + tabs
	for (i=0; i<=13; i+=1)
		Execute "SetVariable setvar" + labels[i] + ",pos={ " + pos[i] + "},size={ " + size1[i] + ",20},limits={0,inf,0},title=\"" + labels[i] + "\",value=root:Data:FileInfo_" + file[i] + "[%Displayed][%" + labels[i] + "]" + tabs
	endfor
	Execute "SetVariable setvarStart,pos={ 20,310},size={150, 20},title=\"from dataset\",limits={0,1000,1},value=root:Data:Starts" + tabs
	Execute "SetVariable setvarEnd,pos={180,310},size={135, 20},title=\"to dataset\",limits={0,1000,1},value=root:Data:Ends" + tabs
	Execute "PopupMenu popupChannel,pos={ 82,360}, title=\"Channel\",value=\"Both;1;2\",bodyWidth=60,proc=SelectChannel" + tabs
	Execute "Button buttonLoad,pos={345,320},size={140,30},proc=CallCreateHistograms,title=\"Create Histograms\",fColor=" + buttons + tabs
	
	//******************************** IRF, Single, Double, Triple Tabs ***************************************************//	
	Make/O numvars = {7,8,10,12}
	Make/O/T type = {"IRF", "S", "D", "T"}
	Make/O/T file = {"IRF", "Single", "Double", "Triple"}
	Make/O/T vars = {"S", "tg", "sigma", "deltat", "rho", "theta1", "theta2", "b", "tau1", "tau2", "bet", "tau3", "gamm"}
	Make/O/T labels = {"S", "\Z12t\Bg", "\F'Symbol's", "\F'Symbol'D\F'Arial't", "\F'Symbol'r", "\Z12\F'Symbol'q\B1", "\Z12\F'Symbol'q\B2", "b", "\Z12\F'Symbol't\B1", "\Z12\F'Symbol't\B2", "\F'Symbol'b", "\Z12\F'Symbol't\B3", "\F'Symbol'g"}
	Make/O/T limits = {"1000", "0.01", "0.001", "0.01", "0.01", "0.01", "0.01", "100", "0.1", "0.1", "0.1", "0.1", "0.1"}
	Make/O size = {245, 280, 330, 380}			// height of group...Man GroupBox
	
	for (	i=0; i<=3; i+=1)		// loop through to create IRF, Single, Double, and Triple tabs
		tabs = font + ",userdata(tabnum)=  \"" + num2str(i+1) + "\",userdata(tabcontrol)=  \"AnalysisTabs\",disable=1"
		String tabs2a = font + ",userdata(tabnum)=  \"0\",userdata(tabcontrol)=  \"" + type[i] + "Tabs\",disable=1"
		String tabs2b = font + ",userdata(tabnum)=  \"1\",userdata(tabcontrol)=  \"" + type[i] + "Tabs\",disable=1"
		// left side of panel
		Execute "GroupBox group" + type[i] + "Man,pos={12,90},size={342," + num2str(size[i]) + "}, title=\"Start Values for Fit\",frame=0" + tabs
		Execute "TitleBox title" + type[i] + "Text1,pos={153,115},title=\"Fix\",frame=0" + tabs
		Execute "TitleBox title" + type[i] + "Text2,pos={183,115},title=\"Lower Limit\",frame=0" + tabs
		Execute "TitleBox title" + type[i] + "Text3,pos={268,115},title=\"Upper Limit\",frame=0" + tabs
		if (i > 0)		// coloured GroupBoxes don't exist for IRF tab
			Execute "GroupBox group" + type[i] + "Line1,pos={18,335},size={330," + num2str(30+50*(i-1)) + "},title=\"\",frame=0,labelBack=" + line + tabs
			Execute "GroupBox group" + type[i] + "Line2,pos={375,335},size={220," + num2str(30+50*(i-1)) + "},title=\"\",frame=0,labelBack=" + line + tabs2a						
		endif
		if (i==2)		// average tau (alpha) and alpha values are only displayed for double tab
			Execute "SetVariable setvarDalphaFit,pos={23,425},size={120,20},limits={-inf,inf,0}, title=\"\F'Symbol'a\",value=root:Analysis:Fits:FitValuesDouble[%Displayed][%alpha]" + tabs
			Execute "SetVariable setvarDalphaError,pos={153,425},size={90,20},limits={-inf,inf,0}, title=\"\",value=root:Analysis:Fits:FitValuesDouble[%Displayed][%alphaError]"+ tabs
			Execute "SetVariable setvarDavgtaub,pos={23,450},size={120,20},limits={-inf,inf,0}, title=\"\Z12\F'Symbol'<t>\Bb\",value=root:Analysis:Fits:FitValuesDouble[%Displayed][%avgtau_beta]" + tabs
			Execute "SetVariable setvarDavgtaubError,pos={153,450},size={90,20},limits={-inf,inf,0}, title=\"\",value=root:Analysis:Fits:FitValuesDouble[%Displayed][%avgtau_betaError]" + tabs
			Execute "SetVariable setvarDavgtaua,pos={23,475},size={120,20},limits={-inf,inf,0}, title=\"\Z12\F'Symbol'<t>\Ba\",value=root:Analysis:Fits:FitValuesDouble[%Displayed][%avgtau_alpha]" + tabs
			Execute "SetVariable setvarDavgtauaError,pos={153,475},size={90,20},limits={-inf,inf,0}, title=\"\",value=root:Analysis:Fits:FitValuesDouble[%Displayed][%avgtau_alphaError]" + tabs	
		endif
		if (i==3)	// average tau (beta) displayed for double and triple exp fits
			Execute "SetVariable setvarTavgtaub,pos={23,475},size={120,20},limits={-inf,inf,0}, title=\"\Z12\F'Symbol'<t>\Bb\",value=root:Analysis:Fits:FitValuesTriple[%Displayed][%avgtau_beta]" + tabs
			Execute "SetVariable setvarTavgtaubError,pos={153,475},size={90,20},limits={-inf,inf,0}, title=\"\",value=root:Analysis:Fits:FitValuesTriple[%Displayed][%avgtau_betaError]" + tabs
		endif
		// right side of panel
		Execute "TabControl " + type[i] + "Tabs,pos={360,85},size={250," + num2str(size[i]+70) + "},proc=ACL_DisplayTab, labelBack=" + sub + tabs
		Execute "TabControl " + type[i] + "Tabs,userdata(currenttab)=  \"0\",tabLabel(0)=\"Fit Histogram\", value= 0"
		Execute "GroupBox group" + type[i] + "Man2,pos={370,112},size={230," + num2str(size[i]+3) + "}, title=\"Results of Fit\",frame=0" + tabs2a
		Execute "SetVariable setvar" + type[i] + "chi,pos={380," + num2str(size[i]+90) + "},size={105,20},labelBack=" + chi + ",title=\"\Z10\F'Symbol'c\S2\",limits={-inf,inf,0},value=root:Analysis:Fits:FitValues" + file[i] + "[%Displayed][%Chisq]" + tabs2a
		Execute "Button button" + type[i] + "Fit,pos={370," + num2str(size[i]+120) + "},size={40,25},fColor=" + buttons + ",proc=FitTimeHist,title=\"Fit\"" + tabs2a
		Execute "Button button" + type[i] + "Apply,pos={420," + num2str(size[i]+120) + "},size={100,25},fColor=" + buttons + ",proc=ApplyResults,title=\"Apply Results\"" + tabs2a
		if (i>0)
			Execute "Button button" + type[i] + "Cont,pos={530," + num2str(size[i]+120) + "},size={70,25},fColor=" + buttons + ",proc=GetBestFit" + type[i] + ",title=\"Cont. Fit\"" + tabs2a
		endif
		// both sides of panel
		for (k=0; k<=numvars[i]; k+=1)		// loop through to create the SetVariable, CheckBox, etc. for S, tg, sigma, ...
			Execute "SetVariable setvar" + type[i] + vars[k] + ",pos={23," + num2str(135+k*25+(Sign(k-8)+1)/2*5) + "},size={120,22},title=\"" + labels[k] + "\",proc=ShowCurve,value=root:Analysis:Manual:TrialValues" + file[i] + "[%Displayed][%" + vars[k] + "],limits={-inf,inf," + limits[k] + "}" + tabs
			Execute "CheckBox check" + type[i] + "Fix" + vars[k] + ",pos={153," + num2str(135+k*25+(Sign(k-8)+1)/2*5) + "},title=\"\",proc=CheckFix" + tabs
			Execute "SetVariable setvar" + type[i] + "LL" + vars[k] + ",pos={183," + num2str(135+k*25+(Sign(k-8)+1)/2*5) + "},size={75,20},title=\" \",limits={-inf,inf,1000},value=root:Analysis:Manual:TrialValues" + file[i] + "[%Displayed][%LL" + vars[k] + "]" + tabs
			Execute "SetVariable setvar" + type[i] + "UL" + vars[k] + ",pos={268," + num2str(135+k*25+(Sign(k-8)+1)/2*5) + "},size={75,20},title=\" \",limits={-inf,inf,1000},value=root:Analysis:Manual:TrialValues" + file[i] + "[%Displayed][%UL" + vars[k] + "]" + tabs
			Execute "SetVariable setvar" + type[i] + vars[k] + "Fit,pos={380," + num2str(135+k*25+(Sign(k-8)+1)/2*5) + "},size={105,20},title=\"" + labels[k] + "\",limits={-inf,inf,0},value=root:Analysis:Fits:FitValues" + file[i] + "[%Displayed][%" + vars[k] + "]" + tabs2a
			Execute "SetVariable setvar" + type[i] + vars[k] + "Error,pos={495," + num2str(135+k*25+(Sign(k-8)+1)/2*5) + "},size={ 90,20},title=\"\",limits={-inf,inf,0}	,value=root:Analysis:Fits:FitValues" + file[i] + "[%Displayed][%" + vars[k] + "Error]" + tabs2a
		endfor
		if (i==1 || i==2)	// there's an extra tab control on the Single and Double tabs		
			Execute "TabControl " + type[i] + "Tabs,tabLabel(1)=\"Pixel Analysis\", value= 0"
			Execute "SetVariable setvar" + type[i] + "Starts,pos={370,115}, title=\"Analyze\",size={115,20},limits={0,1000,1},value=root:Data:Starts" + tabs2b
			Execute "SetVariable setvar" + type[i] + "Ends,pos={495,115}, title=\"to\",		size={ 80,20},limits={0,1000,1},value=root:Data:Ends" + tabs2b
			Execute "PopupMenu popup" + type[i] + "Binning,pos={460,140}, title=\"Binning\",value=\"1x1;2x2;4x4;8x8;16x16;32x32;64x64;128x128;256x256;512x512\",bodyWidth=80,proc=SetBinning" + tabs2b
			Execute "Button button" + type[i] + "DoIt,pos={370,170},size={70,25},proc=StartSinglePixelAnalysis,title=\"Start\",fColor=" + buttons + tabs2b
			Execute "CheckBox check" + type[i] + "Preprocess,pos={465,165},title=\"Preprocesss\",variable=root:Analysis:Pixels:PreprocessFlag" + tabs2b
			if (i==1)
				Execute "CheckBox check" + type[i] + "Lifetime,pos={465,185},title=\"Single Lifetime\",variable=root:Analysis:Pixels:LifetimeFlag" + tabs2b
			else
				Execute "CheckBox check" + type[i] + "Lifetime,pos={465,185},title=\"Double Lifetime\",variable=root:Analysis:Pixels:LifetimeFlag" + tabs2b
			endif
			Execute "GroupBox group" + type[i] + "Movie,pos={370,210},size={230,105},frame=0,title=\"Time-course\"" + tabs2b
			Execute "SetVariable setvar" + type[i] + "TFrames,pos={380,235},size={180,20},limits={0,inf,0},title=\"Total Frames\",value=root:Data:FileInfo_Variables[%Displayed][%TotalFrames]" + tabs2b
			Execute "SetVariable setvar" + type[i] + "TpF,pos={380,260},size={200,20},limits={0,inf,0},title=\"Time/Frame (sec)\",value=root:Data:FileInfo_Variables[%Displayed][%TimeperFrame]" + tabs2b
			Execute "SetVariable setvar" + type[i] + "Window,pos={380,285},size={210,20},limits={0,1000,1},title=\"Window (Frames):\",value=root:Data:FileInfo_Variables[%Displayed][%TimeWindow]" + tabs2b
			Execute "Button button" + type[i] + "PixelInfo,pos={370,325},size={90,25},proc=GetPixelInfo,title=\"Pixel Info\"" + tabs2b
			Execute "Button button" + type[i] + "PixelInfo,help={\"With cursor A select a pixel in Signal2BackgroundImage or Signal2Background2Image, depending on whether you analyze for 1 or 2 lifetimes.\" },fColor=" + buttons
		endif
	endfor 
	
	//******************************** ROI Tab *************************************************************************************************************//	
	tabs = font + ",userdata(tabnum)=  \"5\",userdata(tabcontrol)=  \"AnalysisTabs\",disable=1"
	Execute "Button buttonMakeROI,pos={20,90},size={100,25},proc=MyCreateImageROIPanel,title=\"Create ROI\",fColor=" + buttons + tabs
	Execute "ListBox listROIs,	pos={260,90},size={150,200},listwave=root:Analysis:ROIMasks:ROIMaskList,frame=4,mode=2,selRow=0,proc=MyROIUpdateProc" + tabs
	Execute "Button buttonROIRecalc,pos={20,120},size={190,25},proc=RecalculateTimeHist,title=\"Recalculate Time Histogram\",fColor=" + buttons + tabs
	Execute "Button buttonCopyROI,pos={20,150},size={150,25},proc=CopyROI,title=\"Copy threshold ROI\",fColor=" + buttons + tabs
	Execute "Button buttonAppend,pos={20,175},size={150,25},proc=AppendToImages,title=\"Append To Images\",fColor=" + buttons + tabs
	
	KillWaves/Z numvars, type, file, vars, labels, limits, size, pos, size1		// kill the temporary waves we had to use
End

//*************************************************************************************//
//******************************* Functions called from the panel *******************************//
//*************************************************************************************//

// Load Files *********************************
Function SelectArrivalTimeFile(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Data
	NVAR MaxData, Show
	Variable N, i, sep
	String file
	Multiopenfiles
	if (V_Flag == 0)	// User selected files
		N = ItemsInList(S_FileName)	// find out how many files were selected
		if (MaxData == 0)
			Make/T/N=(N,2)/O FileTable
		else
			Wave/T FileTable
			Redimension/N=(numpnts(FileTable)+N,2) FileTable
		endif
		// first fill the FileTable
		for (i = 0; i < N; i += 1)
			file = StringFromList(i, S_FileName, ";")
			if (stringmatch(file, "*.pt3") == 0)		// no match
				DoAlert 0, "The selected file \"" + file + "\" is not a .pt3 file and will be skipped."
			else 
				FileTable[MaxData][0] = file
				Show = MaxData
				MaxData += 1	
				// retrieve file information
				pFLIM_Picoharp Show
				Wave FileInfo_Variables
				Wave/T FileInfo_Strings
				FileInfo_Variables[Show][%xSize] = V_xsize
				FileInfo_Variables[Show][%ySize] = V_ysize
				FileInfo_Variables[Show][%TotalCounts] = V_counts
				if (V_frames == 0)	// can happen if scan got aborted during the first frame
					V_frames = 1
				endif
				FileInfo_Variables[Show][%TotalFrames] = V_frames
				FileInfo_Variables[Show][%Dwelltime] = V_dwelltime 
				FileInfo_Variables[Show][%Binsize] = V_binsize	
				FileInfo_Variables[Show][%BinsUsed] = V_binsused
				FileInfo_Variables[Show][%SyncPeriod] = V_syncperiod
				FileInfo_Strings[Show][%ScanPattern] = S_pattern
				FileInfo_Strings[Show][%MeasurementMode] = S_mode
				FileInfo_Strings[Show][%Filetime] = S_filetime
				FileInfo_Strings[Show][%CommentField] = S_comment
				DisplayData ("",0,"","")
			endif
			SetDataFolder root:Data
		endfor
		// change the limits of the SetVariable setvarWave
		SetVariable setvarWave limits={0,MaxData-1,1}, win=FLIMPanel
		SetVariable setvarSStarts, limits={0,MaxData-1,1}, win=FLIMPanel
		SetVariable setvarSEnds, limits={0,MaxData-1,1}, win=FLIMPanel
		SetVariable setvarDStarts, limits={0,MaxData-1,1}, win=FLIMPanel
		SetVariable setvarDEnds, limits={0,MaxData-1,1}, win=FLIMPanel
		DoUpdate
	endif
End

// CallCreateHistograms *******************************************
Function CallCreateHistograms(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Data
	NVAR Show, Starts, Ends
	Wave FileInfo_Variables
	Wave IRF = root:Analysis:Manual:TrialValuesIRF
	Wave Single = root:Analysis:Manual:TrialValuesSingle
	Wave Double = root:Analysis:Manual:TrialValuesDouble
	Wave Triple = root:Analysis:Manual:TrialValuesTriple
	Variable i, TotalFrames, StartFrame, EndFrame, Channel, BinsUsed, Binsize, SyncPeriod, counts, k
	for (i = Starts; i <= Ends; i+=1 )
		Show = i
		TotalFrames = FileInfo_Variables[i][%TotalFrames]
		StartFrame = FileInfo_Variables[i][%StartFrame]
		EndFrame = FileInfo_Variables[i][%EndFrame]
		Binsused = FileInfo_Variables[i][%BinsUsed]
		Binsize = FileInfo_Variables[i][%Binsize]
		Syncperiod = FileInfo_Variables[i][%SyncPeriod]
		Channel = FileInfo_Variables[i][%Channel]
		if (EndFrame >= totalframes || EndFrame < 1)
			EndFrame = totalframes - 1
		endif
		pFLIM_Histogram/F={StartFrame, EndFrame}/B=(binsused)/C=(Channel) i		
		// get time per frame
		Variable totaltime = syncperiod * V_totalsyncs
		FileInfo_Variables[i][%TimeperFrame] = totaltime / (EndFrame - StartFrame + 1) / 1e9	// also convert to sec
		// correct wave scaling
		Wave hist = $("hist_" + num2str(i))
		SetScale/P x, 0, binsize, hist
		// Create the weighting wave
		Duplicate/O hist, $("weigh_" + num2str(i))
		Wave weigh = $("weigh_" + num2str(i))
		weigh = Sqrt(weigh)-1
		// now calculate how many photons are actually included in the histogram (some were recorded in bin > 1562)
		counts = 0
		for (k = 0; k < Binsused; k+= 1)
			counts += hist[k]
		endfor
		FileInfo_Variables[i][%TotalCounts] = counts
		IRF[i][%S] = counts
		Single[i][%S] = counts
		Double[i][%S] = counts
		Triple[i][%S] = counts
	endfor
	Show = Ends
	Beep
	DisplayData ("",0,"","")
End

// RecalculateTimeHist *******************************************
Function RecalculateTimeHist(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Data
	NVAR Show, Starts, Ends
	Wave FileInfo_Variables
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	Variable i, TotalFrames, StartFrame, EndFrame, Channel, BinsUsed, Binsize
	TotalFrames = FileInfo_Variables[Show][%TotalFrames]
	StartFrame = FileInfo_Variables[Show][%StartFrame]
	EndFrame = FileInfo_Variables[Show][%EndFrame]
	Binsused = FileInfo_Variables[Show][%BinsUsed]
	Binsize = FileInfo_Variables[Show][%Binsize]
	if (EndFrame >= totalframes || EndFrame < 1)
		EndFrame = totalframes - 1
	endif
	ControlInfo/W=FLIMpanel listROIs	
	SetDataFolder root:Analysis:ROIMasks
	Wave/T MaskList = ROIMaskList
	if (V_Value > 0)
		String Mask = "root:Analysis:ROIMasks:" + MaskList[V_Value]
		Wave MaskUsed = $Mask
	else
		Mask = "root:Analysis:ROIMasks:" + MaskList[1]
		Duplicate/O $Mask, NoMask
		Wave NoMask
		NoMask = 0
		Wave MaskUsed = NoMask
	endif
	
	SetDataFolder root:Data
	String Histname = "hist_" + num2str(Show)
	DoWindow/K TimeHist
	KillWaves/Z $Histname
//	pFLIM_RedoHist/M=MaskUsed/F={StartFrame, EndFrame}/B=(binsused)/C=(Channel) Show	
	pFLIM_Histogram/M=MaskUsed/F={StartFrame, EndFrame}/B=(binsused)/C=(Channel) Show		
	// Create the weighting wave
	Wave hist = $Histname
	SetScale/P x, 0, binsize, hist
	Duplicate/O hist, $("weigh_" + num2str(Show))
	Wave weigh = $("weigh_" + num2str(Show))
	weigh = Sqrt(weigh)
	DisplayData ("",0,"","")
End

// DisplayData *********************************
Function DisplayData (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	// value of variable as number
	String varStr		// value of variable as string
	String varName		// name of variable
	PauseUpdate; Silent 1
	SetDataFolder root:Data
	NVAR Show, Starts, Ends
	Wave FileInfo_Variables
	Wave/T FileTable, CurrName, FileInfo_Strings
	SetDataFolder root:Analysis:Manual
	Wave TrialValuesIRF, TrialValuesSingle, TrialValuesDouble, TrialValuesTriple
	SetDataFolder root:Analysis:Fits
	Wave FitValuesIRF, FitValuesSingle, FitValuesDouble, FitValuesTriple
	Variable i
	// since the Filepath string is very long, we need to split it up
	String one="", two=""
	if (WaveExists(FileTable))
		sscanf FileTable[Show], "%65[A-Za-z0-9 :_-/.\]%75[A-Za-z0-9 :_-/.\]", one, two
		CurrName[0] = {one, two}
	endif
	Starts = Show
	Ends = Show
	
	String histname = "root:Data:hist_" + num2str(Show)	
	if (waveexists($histname) ) 
		DisplayTimeHist("",1)
	endif
	
	// show the correct panel
	STRUCT WMTabControlAction ms
	ms.ctrlName =  "AnalysisTabs"
	ms.win =  "FLIMpanel"
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	ms.eventCode = 2
	ms.tab = Option
	ACL_DisplayTab(ms)
	DoUpdate
	
	// update the list with ROIs
	SetDataFolder root:Analysis:ROIMasks
	Wave/T ROIMaskTable, ROIMaskList
	ROIMaskList = ROIMaskTable[p][Show]
	
	// update the Fix checkboxes
	CheckFixUpdatePanel()
	
	// update the Binning pop-up
	BinningUpdatePanel()
		
	// update the panel
	SetDataFolder root:Data:
	for (i = 0; i < DimSize(FileInfo_Variables, 0); i+=1)
		SetDataFolder root:Data
		SetDimLabel 0,i,$"", FileInfo_Variables, FileInfo_Strings
		SetDataFolder root:Analysis:Manual
		SetDimLabel 0,i,$"", TrialValuesIRF, TrialValuesSingle, TrialValuesDouble, TrialValuesTriple
		 SetDataFolder root:Analysis:Fits
		SetDimLabel 0,i,$"", FitValuesIRF, FitValuesSingle, FitValuesDouble, FitValuesTriple
	endfor
	SetDataFolder root:Data
	SetDimLabel 0,Show,Displayed, FileInfo_Variables, FileInfo_Strings
	SetDataFolder root:Analysis:Manual
	SetDimLabel 0,Show,Displayed, TrialValuesIRF, TrialValuesSingle, TrialValuesDouble, TrialValuesTriple
	SetDataFolder root:Analysis:Fits
	SetDimLabel 0,Show,Displayed, FitValuesIRF, FitValuesSingle, FitValuesDouble, FitValuesTriple
	
	ControlUpdate/A/W=FLIMpanel
	
	// Show Images
	DisplayIntensity("",1)
	DisplayPixelAnalysisResults("",1)
	DisplayWeighted("",1)
	DoUpdate
	DoWindow/F FLIMPanel 
	SetDataFolder root:Data
End

// ShowCurves *********************************
Function ShowCurve(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	// value of variable as number
	String varStr		// value of variable as string
	String varName		// name of variable
	NVAR Show=root:Data:Show
	Wave SelectedTab = root:Analysis:SelectedTab
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	Variable SyncPeriod = FileInfo_Variables[Show][%SyncPeriod]
	Variable BinSize = FileInfo_Variables[Show][%BinSize]
	Variable Option = SelectedTab[Show] 
	SetDataFolder root:Analysis:Manual
	DoWindow/F TimeHist 
	if(V_Flag==0)	// if there is no window
		DisplayData("", 0, "", "")
	endif
	Wave Hist = $("root:Data:hist_" + num2str(Show))
	if (WaveExists(Hist))
		Duplicate/O Hist, TestWave, Tau1Wave, Tau2Wave, Tau3Wave
		Duplicate/O TestWave, Gaussian, C1, C2, Back
		Make/O/T Name = {"", "IRF", "Single", "Double","Triple"}	
		// now we have to find out what the current tab is. Look at ctrlName: setvarIRF..., setvarS..., or setvarD...?
		String AppendName, Gaussname, C1name, C2name, Bname
		if (Option == 1)
			Wave Val = TrialValuesIRF
			SetDataFolder root:Analysis:Manual:IRF
		elseif (Option == 2)
			Wave Val = TrialValuesSingle
			SetDataFolder root:Analysis:Manual:Single
		elseif (Option == 3)
			Wave Val = TrialValuesDouble
			SetDataFolder root:Analysis:Manual:Double
		elseif (Option == 4)
			Wave Val = TrialValuesTriple
			SetDataFolder root:Analysis:Manual:Triple
		endif
		if (Option == 1)
			CalculateIRF(Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], Val[Show][%b], TestWave)
			CalculateComponents(1, Val[Show][%tg], Val[Show][%sigma], Val[Show][%tg]+Val[Show][%deltat], Val[Show][%rho],  Val[Show][%theta1], Val[Show][%theta2], Val[Show][%b], Gaussian, C1, C2, Back)	
		elseif (Option == 2)		//a, 				tg,				 sigma,					 deltat, 				rho,				 theta1,				theta2, 	  bet,			 tau1, tau2, gamm, tau3, 	b, 	SyncPeriod, BinSize, His	
			Calc3Lifetimes(Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], 0, Val[Show][%tau1], 1, 0, 1, Val[Show][%b], SyncPeriod, BinSize, TestWave)
			CalculateComponents(1,Val[Show][%tg], Val[Show][%sigma], Val[Show][%tg]+Val[Show][%deltat], Val[Show][%rho],  Val[Show][%theta1], Val[Show][%theta2], Val[Show][%b], Gaussian, C1, C2, Back)
		elseif (Option == 3)
			Calc3Lifetimes(Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], Val[Show][%bet], Val[Show][%tau1], Val[Show][%tau2], 0, 1, Val[Show][%b], SyncPeriod, BinSize, TestWave)
			Calc3Lifetimes((1-Val[Show][%bet])*Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], 0, Val[Show][%tau1], Val[Show][%tau2], 0, 1, Val[Show][%b], SyncPeriod, BinSize, Tau1Wave)
			Calc3Lifetimes(Val[Show][%bet]*Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], 1, Val[Show][%tau1], Val[Show][%tau2], 0, 1, Val[Show][%b], SyncPeriod, BinSize, Tau2Wave)
			CalculateComponents(1, Val[Show][%tg], Val[Show][%sigma], Val[Show][%tg]+Val[Show][%deltat], Val[Show][%rho],  Val[Show][%theta1], Val[Show][%theta2], Val[Show][%b], Gaussian, C1, C2, Back)
		elseif (Option == 4)
			Calc3Lifetimes(Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], Val[Show][%bet], Val[Show][%tau1], Val[Show][%tau2], Val[Show][%gamm], Val[Show][%tau3], Val[Show][%b], SyncPeriod, BinSize, TestWave)
			Calc3Lifetimes((1-Val[Show][%bet])*Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], 0, Val[Show][%tau1], Val[Show][%tau2], 0, Val[Show][%tau3], Val[Show][%b], SyncPeriod, BinSize, Tau1Wave)
			Calc3Lifetimes(Val[Show][%bet]*Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], 1, Val[Show][%tau1], Val[Show][%tau2], 0, Val[Show][%tau3], Val[Show][%b], SyncPeriod, BinSize, Tau2Wave)
			Calc3Lifetimes(Val[Show][%bet]*Val[Show][%S], Val[Show][%tg], Val[Show][%sigma], Val[Show][%deltat], Val[Show][%rho], Val[Show][%theta1], Val[Show][%theta2], 0, Val[Show][%tau1], Val[Show][%tau2], 1, Val[Show][%tau3], Val[Show][%b], SyncPeriod, BinSize, Tau3Wave)
			CalculateComponents(1, Val[Show][%tg], Val[Show][%sigma], Val[Show][%tg]+Val[Show][%deltat], Val[Show][%rho],  Val[Show][%theta1], Val[Show][%theta2], Val[Show][%b], Gaussian, C1, C2, Back)	
		endif
		Appendname = Name[Option] + "Init_"+num2str(Show)
		Gaussname = "Gaussian_" + Name[Option] +num2str(Show)
		C1name = "Comp1_" + Name[Option] +num2str(Show)
		C2name = "Comp2_" + Name[Option] +num2str(Show)
		Bname = "Backg_" + Name[Option] +num2str(Show)
		Duplicate/O TestWave, $AppendName			
		Duplicate/O Gaussian, $Gaussname
		Duplicate/O C1, $C1name
		Duplicate/O C2, $C2name
		Duplicate/O Back, $Bname
		if (Option == 3 || Option == 4)
			C1name = "Tau1_" + Name[Option] +num2str(Show)
			C2name = "Tau2_" + Name[Option] +num2str(Show)
			Duplicate/O Tau1Wave, $C1Name 
			Duplicate/O Tau2Wave, $C2Name 
		endif		
		if (Option == 4)
			C1name = "Tau3_" + Name[Option] +num2str(Show)
			Duplicate/O Tau3Wave, $C1Name 
		endif	
	
		// obtain bottom axis range
		String Axes = AxisInfo("TimeHist", "bottom")
		DisplayTimeHist("",1)	// show curves
		// now set bottom axis range to what it was before updating the graph
		Execute/Q/Z StringByKey("SETAXISCMD", Axes)
	endif
	DoWindow/F FLIMPanel 

	KillWaves/Z TestWave, TestCum, Term1, Term2, stgx1, stgx2, Gaussian, C1, C2, Back, Tau1Wave, Tau2Wave, Tau3Wave
	DoUpdate
	DoWindow/F FLIMPanel 	
End

// ApplyResults *********************************
Function ApplyResults (ctrlName) : ButtonControl
	String ctrlName
	NVAR Show=root:Data:Show
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable i, Option = SelectedTab[Show]
	Make/O/T labels={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1", "tau2", "bet", "tau3", "gamm", "alpha"}
	if (Option == 1)
		String Trial ="root:Analysis:Manual:TrialValuesIRF"	
		String Fit ="root:Analysis:Fits:FitValuesIRF"	
	elseif (Option == 2)
		Trial ="root:Analysis:Manual:TrialValuesSingle"	
		Fit ="root:Analysis:Fits:FitValuesSingle"	
	elseif (Option == 3)
		Trial ="root:Analysis:Manual:TrialValuesDouble"	
		Fit ="root:Analysis:Fits:FitValuesDouble"					
	elseif (Option == 4)
		Trial ="root:Analysis:Manual:TrialValuesTriple"	
		Fit ="root:Analysis:Fits:FitValuesTriple"					
	endif
	// Now save parameters in our table	
	for (i = 0; i < numpnts(labels); i += 1)
		Execute (Trial + "[" + num2str(Show) + "][%" + labels[i] + "]=" + Fit + "[" + num2str(Show) + "][%" + labels[i] + "]")
	endfor
	ShowCurve(ctrlName,0,"","")
End

// FitTimeHist *********************************
Function FitTimeHist(ctrlName) : ButtonControl
	String ctrlName
	NVAR Show=root:Data:Show
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	Variable start, ends, i, k=0, N=13
	if (Option == 1)
		SetDataFolder root:Analysis:Manual:IRF	//  first load starting values, limits, ...
		Wave Trial = root:Analysis:Manual:TrialValuesIRF
		String FitName = "root:Analysis:Fits:FitValuesIRF"
		N=8
	elseif (Option == 2)
		SetDataFolder root:Analysis:Manual:Single	//  first load starting values, limits, ...
		Wave Trial = root:Analysis:Manual:TrialValuesSingle
		FitName = "root:Analysis:Fits:FitValuesSingle"
	elseif (Option == 3)
		SetDataFolder root:Analysis:Manual:Double	//  first load starting values, limits, ...
		Wave Trial = root:Analysis:Manual:TrialValuesDouble
		FitName = "root:Analysis:Fits:FitValuesDouble"
	elseif (Option == 4)
		SetDataFolder root:Analysis:Manual:Triple	//  first load starting values, limits, ...
		Wave Trial = root:Analysis:Manual:TrialValuesTriple
		FitName = "root:Analysis:Fits:FitValuesTriple"
	endif	
	CheckLimits(Option)
	Wave Hist = $("root:Data:hist_" + num2str(Show))
	Wave Weigh = $("root:Data:weigh_" + num2str(Show))	
	Variable aExists= strlen(CsrInfo(A, "TimeHist")) > 0
	Variable bExists= strlen(CsrInfo(B, "TimeHist")) > 0			
	if (aExists > 0 && bExists >0)
		start = x2pnt(Hist, xcsr(A, "TimeHist"))
		ends = x2pnt(Hist, xcsr(B, "TimeHist"))
	else
		start = x2pnt(Hist, 0)
		ends = x2pnt(Hist, numpnts(Hist)-2)
	endif	
	Make/O/N=(N) W_coef, Fixed, LLs, ULs
	//find column number of column "FixS"
	for (i = 0; i < DimSize(Trial,1); i += 1)
		String lab = GetDimLabel(Trial, 1, i)
		if (stringmatch(lab, "S") == 1)	//success
			W_coef[] = Trial[Show][p+i]		// wave that contains initial guesses	
			if (Option > 1)	// don't do for IRF
				// add a point at the end containing SyncPeriod
				Redimension/N=(numpnts(W_coef)+1) W_coef
				W_coef[numpnts(W_coef)-1]= FileInfo_Variables[Show][%SyncPeriod]
			endif
		elseif (stringmatch(lab, "FixS") == 1)	//success
			Fixed[] = Trial[Show][p+i]
			if (Option > 1)	// don't do for IRF
				Redimension/N=(numpnts(Fixed)+1) Fixed
				Fixed[numpnts(Fixed)-1]= 1		// Fix SyncPeriod
			endif
		elseif (stringmatch(lab, "LLS") == 1)	//success
			LLs[] = Trial[Show][p+i]
			if (Option > 1)	// don't do for IRF
				Redimension/N=(numpnts(LLs)+1) LLs
				LLs[numpnts(LLs)-1]= 0			// lower limit for SyncPeriod
			endif
		elseif (stringmatch(lab, "ULS") == 1)	//success
			ULs[] = Trial[Show][p+i]
			if (Option > 1)	// don't do for IRF
				Redimension/N=(numpnts(ULs)+1) ULs
				ULs[numpnts(ULs)-1]= 1e14		// upper limit for SyncPeriod
			endif
		endif
	endfor
	if (Option > 1)
		N += 1		// added 1 (SyncPeriod)
	endif
	if (Option == 2)
		W_coef[9] = {1,0,1,0}		// W_coef={0:a, 1:tg, 2:sigma, 3:deltat, 4:rho, 5:theta1, 6:theta2, 7:b, 8:tau1, 9:tau2, 10:beta, 11:tau3, 12:gamma, 13:SyncPeriod}
		Fixed[9] = {1,1,1,1}
	elseif (Option == 3)
		W_coef[11] = {1,0}		// W_coef={0:a, 1:tg, 2:sigma, 3:deltat, 4:rho, 5:theta1, 6:theta2, 7:b, 8:tau1, 9:tau2, 10:beta, 11:tau3, 12:gamma, 13:SyncPeriod}
		Fixed[11] = {1,1}
	endif
	
	String hold =  ""	// string will tell FuncFit which values to hold
	for (i = 0; i < N; i += 1)	
		hold += num2str(Fixed[i])
		if (Fixed[i] > 0)
			k += 1			// counts how many values will be hold
		endif
	endfor
	Variable DegFreedom = N-k
	Make/O/T/N=(2*(N-k)) T_Constraints		// length is determined by how many values are hold, those must be excluded from limts
	k = 0
	for (i = 0; i < N; i += 1)
		if (Fixed[i] == 0)
			T_Constraints[k] =		"K" + num2str(i) + ">" + num2str(LLs[i])
			T_Constraints[k+1] = 	"K" + num2str(i) + "<" + num2str(ULs[i])
			k += 2
		endif
	endfor
	Duplicate/O Hist, temp, res
	res = 0
	if (Option == 1)
		FuncFit/Q/H=hold/M=2/NTHR=0 IRFHist W_coef Hist[start,ends] /D=temp /I=1 /W=Weigh /C=T_Constraints /R=res
	else
		FuncFit/Q/H=hold/M=2/NTHR=0 Three_Lifetimes W_coef Hist[start,ends] /D=temp /I=1 /W=Weigh /C=T_Constraints /R=res
	endif
	Wave W_sigma, M_Covar
	if (Option == 1)
		SetDataFolder root:Analysis:Fits:IRF
		String Name = "IRFFit_" + num2str(Show) 
	elseif (Option == 2)
		SetDataFolder root:Analysis:Fits:Single
		Name = "SingleFit_" + num2str(Show) 
	elseif (Option == 3)
		SetDataFolder root:Analysis:Fits:Double
		Name = "DoubleFit_" + num2str(Show)
	elseif (Option == 4)
		SetDataFolder root:Analysis:Fits:Triple
		Name = "TripleFit_" + num2str(Show)
	endif
	Duplicate/O/R=[start, ends] temp, $Name
	Name = "Res_" + Name
	Duplicate/O res, $Name
	// divide residual by standard deviation
	Name = "Norm" + Name
	res /= Weigh
	Duplicate/O res, $Name	

	Make/O/T labels={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1", "tau2", "bet", "tau3", "gamm"}
	Duplicate/T/O labels, labels2
	labels2 = labels + "Error"	
	Wave Fits = $FitName
	for (i = 0; i < N-1; i += 1)
		Execute (FitName + "[" + num2str(Show) + "][%" + labels[i] + "]=" + num2str(W_coef[i]) )	// save actual parameters
		Execute (FitName + "[" + num2str(Show) + "][%" + labels2[i] + "]=" + num2str(W_sigma[i]) )	// save errors
	endfor
	Fits[Show][%Chisq]= V_chisq/(V_npnts-DegFreedom)	// save chisq
	if (Option == 3)
		Variable alpha = W_coef[10]/W_coef[9] / ((1-W_coef[10])/W_coef[8] + W_coef[10]/W_coef[9])
		Fits[Show][%alpha] =  alpha		// save alpha
		Variable Ualpha = (W_coef[9]/W_coef[10] 							   * W_sigma[10] )^2
		Ualpha += ( (W_coef[10]-1) * W_coef[9] / W_coef[10] / W_coef[8] * W_sigma[8] )^2
		Ualpha += ( (W_coef[10]-1) 											   * W_sigma[9] )^2
		Ualpha += (1-W_coef[10]) * W_coef[9]^2 / W_coef[10] / W_coef[8] * M_Covar[10][8]
		Ualpha += ( W_coef[10]-1) * W_coef[9] / W_coef[10] 				    * M_Covar[10][9]
		Ualpha -= (W_coef[10]-1)^2 * W_coef[9] / W_coef[8] 				    * M_Covar[8][9]
		Ualpha = Sqrt(Ualpha)
		Ualpha *= alpha^2 / W_coef[10] / W_coef[8] 		
		Fits[Show][%alphaError] =  Ualpha
		// avgtau based on beta
		Variable avgtau = (1-W_coef[10]) * W_coef[8] + W_coef[10] * W_coef[9]		// avgtau = (1-beta) tau1 + beta tau2
		Fits[Show][%avgtau_beta] =  avgtau		// save avgtau
		Variable Uavgtau = (W_coef[9]-W_coef[8])^2   		* W_sigma[10]^2		// (tau2-tau1)^2 		* s_beta^2		W_sigma[10]^2 = M_covar[10][10]		
		Uavgtau += (1-W_coef[10])^2  						* W_sigma[8]^2		// (1-beta)^2 			* s_tau1^2				
		Uavgtau += W_coef[10]^2		  					* W_sigma[9]^2		// beta^2 			* s_tau2^2					
		Uavgtau += (W_coef[9]-W_coef[8])*(1-W_coef[10])	* M_Covar[10][8]		// (1-beta)(tau2-tau1) 	* s_beta_tau1^2
		Uavgtau += (W_coef[9]-W_coef[8])* W_coef[10]		* M_Covar[10][9]		// beta (tau2-tau1) 	* s_beta_tau2^2
		Uavgtau += W_coef[10]*(1-W_coef[10])				* M_Covar[8][9]		// beta (1-beta) 		* s_tau1_tau2^2
		Uavgtau = Sqrt(Uavgtau)
		Fits[Show][%avgtau_betaError] =  Uavgtau		// save avgtauError
		// avgtau based on alpha
		Variable denom = W_coef[10]*W_coef[8] + (1-W_coef[10])*W_coef[9]	 // beta tau1 + (1-beta) tau2
		avgtau = W_coef[8]*W_coef[9] / denom								// tau1 tau2 / ( beta tau1 + (1-beta) tau2)
		Fits[Show][%avgtau_alpha] =  avgtau		// save avgtau
		Uavgtau = (1-W_coef[10])^2 * W_coef[9]^4 						* W_sigma[8]^2				// (1-beta)^2 tau2^4				* s_tau1^2
		Uavgtau += W_coef[10]^2 * W_coef[8]^4 						* W_sigma[9]^2				// beta^2 tau1^4					* s_tau2^2
		Uavgtau += W_coef[8]^2 * W_coef[9]^2 * (W_coef[9]-W_coef[8])^2 	* W_sigma[10]^2				// tau1^2 tau2^2 (tau2-tau1)^2 		* s_beta^2
		Uavgtau += W_coef[10]*(1-W_coef[10]) * W_coef[8]^2 * W_coef[9]^2 			* M_Covar[8][9]	// beta (1-beta) tau1^2 tau2^2		* s_tau1_tau2
		Uavgtau += (1-W_coef[10]) * W_coef[8] * W_coef[9]^3 * (W_coef[9]-W_coef[8]) 	* M_Covar[10][8]	// (1-beta) tau1 tau2^3 (tau2-tau1) 	* s_beta_tau1
		Uavgtau += W_coef[10] * W_coef[8]^3 * W_coef[9] * (W_coef[9]-W_coef[8]) 		* M_Covar[10][9]	// beta tau1^3 tau2 (tau2-tau1)		* s_beta_tau2
		Uavgtau /= denom^4
		Uavgtau = Sqrt(Uavgtau)
		Fits[Show][%avgtau_alphaError] =  Uavgtau		// save avgtauError
	elseif (Option == 4)
		// avgtau based on beta
		avgtau = (1-W_coef[10]) * W_coef[8] + W_coef[10] * W_coef[9]		// avgtau = (1-gamma) [(1-beta) tau1 + beta tau2] + gamma tau3
		Fits[Show][%avgtau_beta] =  avgtau		// save avgtau
		Uavgtau = (1-W_coef[10])^2 * (1-W_coef[12])^2 								* W_sigma[8]^2	// tau1
		Uavgtau += W_coef[10]^2 * W_coef[12]^2 									* W_sigma[9]^2	// tau2
		Uavgtau += W_coef[12]^2 													* W_sigma[11]^2	// tau3
		Uavgtau += (1-W_coef[12])^2 * (W_coef[9]-W_coef[8])^2 						* W_sigma[10]^2	// beta
		Uavgtau += (W_coef[11] - W_coef[10]*W_coef[9] - (1-W_coef[10])*W_coef[8])^2 	* W_sigma[12]^2	// gamma
		Uavgtau += W_coef[10] * (1-W_coef[10]) * (1-W_coef[12])^2 														* M_Covar[8][9]	// tau1 tau2
		Uavgtau += (1-W_coef[10]) * W_coef[12] * (1-W_coef[12]) 															* M_Covar[8][11]	// tau1 tau3
		Uavgtau += (1-W_coef[10]) * (1-W_coef[12])^2 * (W_coef[9] - W_coef[8]) 											* M_Covar[8][10]	// tau1 beta
		Uavgtau += (1-W_coef[10]) * (1-W_coef[12]) * (W_coef[11] - W_coef[10]*W_coef[9] - (1-W_coef[10])*W_coef[8]) 			* M_Covar[8][12]	// tau1 gamma
		Uavgtau += W_coef[10] * W_coef[12] * (1-W_coef[12]) 															* M_Covar[9][11]	// tau2 tau3
		Uavgtau += W_coef[10] * (1-W_coef[12])^2 * (W_coef[9]-W_coef[8]) 												* M_Covar[9][10]	// tau2 beta
		Uavgtau += W_coef[10] * (1-W_coef[12]) * (W_coef[11] - W_coef[10]*W_coef[9] - (1-W_coef[10]*W_coef[8])) 			* M_Covar[9][12]	// tau2 gamma
		Uavgtau += W_coef[12] * (1-W_coef[12]) * (W_coef[9]-W_coef[8]) 													* M_Covar[11][10]	// tau3 beta
		Uavgtau += W_coef[12] * (W_coef[11] - W_coef[10]*W_coef[9] - (1-W_coef[10])*W_coef[8]) 							* M_Covar[11][12]	// tau3 gamma
		Uavgtau += (1-W_coef[12]) * (W_coef[9] - W_coef[8]) * (W_coef[11] - W_coef[10]*W_coef[9] - (1-W_coef[10])*W_coef[8]) 	* M_Covar[10][12]	// beta gamma
		Uavgtau = Sqrt(Uavgtau)
		Fits[Show][%avgtau_betaError] =  Uavgtau		// save avgtauError
	endif
	
	String Axes = AxisInfo("TimeHist", "bottom")	// obtain bottom axis range
	DisplayTimeHist("",1)	// Show fits
	Execute/Q/Z StringByKey("SETAXISCMD", Axes)	// now set bottom axis range to what it was before updating the graph
	KillWaves/Z temp, res, T_Constraints, W_coef, W_sigma, Fixed, LLs, ULs, labels, labels2, M_covar
	DoWindow/F FLIMPanel 
End

//******************************************************************************************//
//********************************************** Pixel by pixel fitting ****************************//
//******************************************************************************************//

// StartSinglePixelAnalysis *********************************
Function StartSinglePixelAnalysis(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Data
	NVAR Show, Starts, Ends
	Wave FileInfo_Variables
	SetDataFolder root:Analysis:Pixels
	NVAR PreprocessFlag, LifetimeFlag
	NVAR Perc = root:Analysis:percentFinished
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	Variable i, TotalFrames, xSize, ySize, Binning, StartFrame, EndFrame, Channel, TimeWindow, Counts
	if (PreprocessFlag == 1)	
		for (i = Starts; i <= Ends; i += 1)
			Show = i
			TotalFrames = FileInfo_Variables[i][%TotalFrames]
			xSize = FileInfo_Variables[i][%xSize]
			ySize = FileInfo_Variables[i][%ySize]
			Binning = FileInfo_Variables[i][%Binning]
			StartFrame = FileInfo_Variables[i][%StartFrame]
			EndFrame = FileInfo_Variables[i][%EndFrame]
			Channel = FileInfo_Variables[i][%Channel]
			TimeWindow = FileInfo_Variables[i][%TimeWindow]	
			Counts = FileInfo_Variables[i][%TotalCounts]
			if (EndFrame >= totalframes || EndFrame < 0)
				EndFrame = 0		// make sure EndFrame is not outside the limits
			endif
			if (TimeWindow == 0)
				TimeWindow = TotalFrames
			endif
			String ImageName = "Intensity_B" + num2str(Binning) + "_" + num2str(Show)
			if (WaveExists($ImageName))
				DoWindow/K IntensityImage
				KillWaves $ImageName
			endif
			SetDataFolder root:Data
			pFLIM_Intensity/B=(Binning)/F={StartFrame, EndFrame}/W=(TimeWindow)/C=(Channel)/S={xSize,ySize}/L=(Counts) Show
			Wave image = $ImageName
			MoveWave image, $("root:Analysis:Pixels:" + ImageName)
			// Show Image
			DisplayIntensity("",1)
			Perc += 1/(Ends-Starts+1) 
			ControlUpdate/W=FLIMPanel valdispProgress
		endfor
	endif
	// now do lifetime analysis
	if (LifetimeFlag == 1 && Option > 1)		// Lifetime analysis is checked and we do single or double exp analysis
		for (i = Starts; i <= Ends; i += 1)
			Show = i
			if (Option > 1)	// single or double
				FitEachPixel(ctrlName)
			endif
		endfor
		DoUpdate
	endif
	Perc = 0
	ControlUpdate/W=FLIMPanel valdispProgress
	DoWindow/F FLIMPanel 	
End

// FitEachPixel *********************************
Function FitEachPixel(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Data
	NVAR Show, Starts, Ends
	Wave FileInfo_Variables
	Wave/T FileTable	
	SetDataFolder root:Analysis:Pixels
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	Variable i, TotalFrames, Binning, StartFrame, EndFrame, Channel, TimeWindow, SyncPeriod, Binsize, k
	
	TotalFrames = FileInfo_Variables[Show][%TotalFrames]
	Binning = FileInfo_Variables[Show][%Binning]
	StartFrame = FileInfo_Variables[Show][%StartFrame]
	EndFrame = FileInfo_Variables[Show][%EndFrame]
	Channel = FileInfo_Variables[Show][%Channel]
	TimeWindow = FileInfo_Variables[Show][%TimeWindow]
	SyncPeriod = FileInfo_Variables[Show][%SyncPeriod]
	Binsize = FileInfo_Variables[Show][%Binsize]
	
	// create "Sorted" string
	String original = FileTable[Show], fullsorted
	String with = "_sort_B" + num2str(Binning) + ".bin"
	fullsorted = ReplaceString(".pt3", original, with)
	
	if (EndFrame >= TotalFrames || EndFrame < 1)
		EndFrame = TotalFrames - 1
	endif
	if (TimeWindow == 0)
		TimeWindow = TotalFrames
	endif
			
	// load all relevant waves here into the function so we can access them
	Wave Hist = $("root:Data:hist_"+ num2str(Show))
	Wave Intensity = $("Intensity_B" + num2str(Binning) + "_"  + num2str(Show))
	String SName = "Signal_B" + num2str(Binning) + "_" + num2str(Show)
	String AlphaName = "Alpha_B" + num2str(Binning ) + "_" + num2str(Show)
	String AvgName = "AvgTau_B" + num2str(Binning ) + "_" + num2str(Show)
	if (!WaveExists(Intensity))			// check that Preprocessing has been done
		DoAlert 0, "Please press the button 'Preprocess' first!"
		Abort
	endif
	
	Variable Npointshist = numpnts(Hist)			
	// Manually make the cumulative distribution
	Make/D/O/N=(Npointshist+1) Hist1Array, Cum1Array, Hist2Array, Cum2Array, BArray	// create waves that will hold the histogram, cumulative, and background	
	SetScale/P x, 0, deltax(hist), Hist1Array, Cum1Array, Hist2Array, Cum2Array, BArray	// set the correct x-scaling				
	
	i = Show	// to shorten the following lines	
	if (Option == 2)	// single
		Wave Trial = root:Analysis:Manual:TrialValuesSingle
		Calc3Lifetimes(Trial[i][%S], Trial[i][%tg], Trial[i][%sigma], Trial[i][%deltat], Trial[i][%rho], Trial[i][%theta1], Trial[i][%theta2], 0, Trial[i][%tau1], 1, 0, 1, 0, SyncPeriod, Binsize, Hist1Array)	// calculate the histogram for tau1
	else					// double
		Wave Trial = root:Analysis:Manual:TrialValuesDouble				
		Calc3Lifetimes(Trial[i][%S], Trial[i][%tg], Trial[i][%sigma], Trial[i][%deltat], Trial[i][%rho], Trial[i][%theta1], Trial[i][%theta2], 0, Trial[i][%tau1], Trial[i][%tau2], 0, 1, 0, SyncPeriod, Binsize, Hist1Array)	// non-FRET fraction		
		Calc3Lifetimes(Trial[i][%S], Trial[i][%tg], Trial[i][%sigma], Trial[i][%deltat], Trial[i][%rho], Trial[i][%theta1], Trial[i][%theta2], 1, Trial[i][%tau1], Trial[i][%tau2], 0, 1, 0, SyncPeriod, Binsize, Hist2Array) // FRET fraction
	endif

	Cum1Array[0] = Hist1Array[0]	// Double exp
	for (i = 1; i <= Npointshist; i += 1)
		Cum1Array [i] = Hist1Array[i] + Cum1Array[i-1]
	endfor
	Cum1Array[] /= Cum1Array[Npointshist]		//normalize
	
	Cum2Array[0] = Hist2Array[0]	// double exp
	for (i = 1; i <= Npointshist; i += 1)
		Cum2Array [i] = Hist2Array[i] + Cum2Array[i-1]
	endfor
	Cum2Array[] /= Cum2Array[Npointshist]		//normalize
	
	BArray = Trial[Show][%b] * p
	BArray/= BArray[Npointshist]			  	//normalize
	
	// create images that will hold the results			
	Variable xSize = DimSize(Intensity, 0)
	Variable ySize = DimSize(Intensity, 1)
	Variable zSize = DimSize(Intensity, 2)
	if (zSize == 1)
		Make/O/N=(xSize, ySize) $SName=0
		Make/O/N=(xSize, ySize)/D MaxFid=nan
	else
		Make/O/N=(xSize, ySize, zSize) $SName=0
		Make/O/N=(xSize, ySize, zSize)/D MaxFid=nan
	endif
	Wave Signal = $SName
	if (Option == 3)
		if (zSize == 1)
			Make/O/N=(xSize, ySize) $AlphaName=nan,  $AvgName=nan
		else
			Make/O/N=(xSize, ySize, zSize) $AlphaName=nan,  $AvgName=nan
		endif
		Wave Alpha = $AlphaName	// until further down this initially contains the beta values
		Wave Avg = $AvgName
		pFLIM_SinglePixel /B=(Binning) /F={StartFrame, EndFrame} /W=(TimeWindow)/C=(Channel) /I={Intensity, Signal, Cum1Array, BArray, Alpha, Cum2Array} /ML=MaxFid /S=fullsorted	
	else
		pFLIM_SinglePixel /B=(Binning) /F={StartFrame, EndFrame} /W=(TimeWindow)/C=(Channel) /I={Intensity, Signal, Cum1Array, BArray} /ML=MaxFid /S=fullsorted	
	endif
	// change signal image to contain counts from signal
	Signal *= Intensity
	MaxFid *= -1
	if (Option == 3)
		// in beta image change pixel that are < 1 in signal to Nan
		for (i = 0; i < xSize; i += 1)
			for (k = 0; k < ySize; k += 1)
				if (Signal[i][k] < 1)
					Alpha[i][k] = Nan
				endif
			endfor
		endfor
		// create avgTau image
		Avg = (1-Alpha) * Trial[Show][%tau1] + Alpha * Trial[Show][%tau2]
		// create alpha image
		Alpha = Alpha/Trial[Show][%tau2] / ( (1-Alpha)/Trial[Show][%tau1] + Alpha/Trial[Show][%tau2] )
		// create the weighted images
		CreateWeighted()
		DisplayWeighted("",1)
	endif
	// save parameter values in note attached to the image
	Make/O/T param={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1", "tau2", "beta"}
	Make/O/N=14 values
	values[] = Trial[Show][p] 
	String s = ""
	for (i = 0; i < numpnts(values); i += 1)
		s = s + param[i] + "=" + num2str(values[i]) + ";"
	endfor
	s += "Window(Frames)=" + num2str(TimeWindow) + ";"
	Note Signal, s		// this attaches the note	
	// finally delete all the waves that were temporarily created in this function
	KillWaves/Z Cum1Array, Cum2Array, Hist1Array, Hist2Array, BArray, param, values, MaxFid
	DisplayPixelAnalysisResults("",1)	// Show Images
	DoWindow/F FLIMPanel 
End

// CreateWeighted *********************************
Function CreateWeighted()
	NVAR Show=root:Data:Show
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	SetDataFolder root:Analysis:Pixels
	Variable h, i, k, ReverseColor
	Variable Binning = FileInfo_Variables[Show][%Binning]
	Variable xSize = FileInfo_Variables[Show][%xSize] / Binning
	Variable ySize = FileInfo_Variables[Show][%ySize] / Binning
	String BetaNameW, AlphaNameW, AvgNameW
	Wave Signal = $("Signal_B" + num2str(Binning) + "_"  + num2str(Show))
	Wave Alpha = $("Alpha_B" + num2str(Binning ) + "_" + num2str(Show))
	AlphaNameW = "Alpha_B" + num2str(Binning ) + "_W_" + num2str(Show)

	Variable layers = DimSize(Signal, 2)
	if (layers == 0)
		layers = 1
	endif
	// first convert the grey image to a 3-layer RGB image using the Rainbow color table
	ColorTab2Wave Rainbow		// extracts colors from the built-in color table and places them in an Nx3 matrix of red, green and blue columns 
	Wave M_colors
	NVAR alpha0= root:Analysis:Pixels:DisplayLowerAlpha
	NVAR alpha1 = root:Analysis:Pixels:DisplayUpperAlpha
	SetScale/I x, alpha0, alpha1, M_colors
	Make/O/N=(xSize, ySize, 3) temp										// make image with 3 layers for RGB	
	// for the alpha image
	Make/O/N=(xSize, ySize, 3, layers ) $AlphaNameW		// 3 layers for RGB, the 4th dimension contains the frames
	Wave AlphaW = $AlphaNameW
	for (h = 0; h < layers ; h += 1)	// go through each frame
		for(i=0; i < xSize; i+= 1)		// go thru rows
			for(k=0; k < ySize; k += 1)	// go thru columns
				temp[i][k][] = M_colors[x2pnt(M_colors, Alpha[i][k][h])][r]
			endfor
		endfor
		ImageTransform/U rgb2hsl temp		// now convert this RGB image to an HSL image (hue, saturation, light)
		Wave M_RGB2HSL						// result of the transformation	
		WaveStats/Q/M=1 Signal	
		M_RGB2HSL[][][2] = Signal[p][q][h]/V_max * 65535		// now replace the light layer with the Signal image and normalize
		ImageTransform hsl2rgb M_RGB2HSL		// finally convert this back to RGB
		Wave M_HSL2RGB	
		AlphaW[][][][h] = M_HSL2RGB[p][q][r]
	endfor
	
	// and also for the avgTau image
	if (WaveExists($("AvgTau_B" + num2str(Binning ) + "_" + num2str(Show))))
		Wave Avg = $("AvgTau_B" + num2str(Binning ) + "_" + num2str(Show))
		AvgNameW = "AvgTau_B" + num2str(Binning ) + "_W_" + num2str(Show)
		ColorTab2Wave Rainbow		// extracts colors from the built-in color table and places them in an Nx3 matrix of red, green and blue columns 
		Wave M_colors
		NVAR tau2= root:Analysis:Pixels:DisplayLowerTau
		NVAR tau1 = root:Analysis:Pixels:DisplayUpperTau
		SetScale/I x, tau1, tau2, M_colors
		Make/O/N=(xSize, ySize, 3, layers ) $AvgNameW		// 3 layers for RGB, the 4th dimension contains the frames
		Wave AvgW = $AvgNameW
		for (h = 0; h < layers ; h += 1)	// go through each frame
			for(i=0; i < xSize; i+= 1)		// go thru rows
				for(k=0; k < ySize; k += 1)	// go thru columns
					temp[i][k][] = M_colors[x2pnt(M_colors, Avg[i][k][h])][r]
				endfor
			endfor
			ImageTransform/U rgb2hsl temp		// now convert this RGB image to an HSL image (hue, saturation, light)	
			M_RGB2HSL[][][2] = Signal[p][q][h]/V_max * 65535		// now replace the light layer with the Signal image and normalize
			ImageTransform hsl2rgb M_RGB2HSL		// finally convert this back to RGB
			AvgW[][][][h] = M_HSL2RGB[p][q][r]
		endfor
	endif
	KillWaves/Z temp, M_RGB2HSL, M_HSL2RGB, M_colors
End

// GetPixelInfo *********************************
Function GetPixelInfo(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Data
	NVAR Show, Starts, Ends
	Wave FileInfo_Variables
	Wave/T FileTable	
	SetDataFolder root:Analysis:Pixels
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	Variable i, Binning, StartFrame, EndFrame, Channel, TimeWindow, BinSize, TotalFrames, SyncPeriod
	Variable a, tg, sigma, deltat, rho,  theta1, theta2, b, tau1, tau2=1, layer	
	
	TotalFrames = FileInfo_Variables[Show][%TotalFrames]
	Binning = FileInfo_Variables[Show][%Binning]
	StartFrame = FileInfo_Variables[Show][%StartFrame]
	EndFrame = FileInfo_Variables[Show][%EndFrame]
	Channel = FileInfo_Variables[Show][%Channel]
	TimeWindow = FileInfo_Variables[Show][%TimeWindow]
	Binsize = FileInfo_Variables[Show][%Binsize]
	SyncPeriod = FileInfo_Variables[Show][%SyncPeriod]
	
	if (EndFrame >= TotalFrames || EndFrame < 1)
		EndFrame = TotalFrames - 1
	endif
	if (TimeWindow == 0)
		TimeWindow = TotalFrames
	endif	
	
	Wave hist = $("root:Data:hist_" + num2str(Show))
	Wave Intensity = $("Intensity_B" + num2str(Binning) + "_"  + num2str(Show))
	Wave Signal = $("Signal_B" + num2str(Binning ) + "_" + num2str(Show))
	if (Option == 2)	// single
		Wave Trial = root:Analysis:Manual:TrialValuesSingle
		Make/O/T param={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1"}
	else
		Wave Trial = root:Analysis:Manual:TrialValuesDouble
		Wave AlphaW = $("Alpha_B" + num2str(Binning ) + "_" + num2str(Show))
		Make/O/T param={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1", "tau2"}	
	endif	
	
	// create "Sorted" string
	String original = FileTable[Show], fullsorted
	String with = "_sort_B" + num2str(Binning) + ".bin"
	fullsorted = ReplaceString(".pt3", original, with)
	
	DoWindow/F IntensityImage
	if (V_Flag == 1 && strlen(CsrInfo(A, "IntensityImage")) != 0)	// if the window exists and the cursor is on it
		Variable pixelx = xcsr(A), pixely = vcsr(A)
		if (DimSize(Intensity, 2) <= 1)
			layer = 0
		else
			NVAR gLayer = root:Packages:WM3DImageSlider:BetaImage:gLayer
			layer = gLayer
		endif
	
	else
		DoWindow/F AlphaImage
		if (V_Flag == 1 && strlen(CsrInfo(A, "AlphaImage")) != 0)		// if the window exists and the cursor is on it
			pixelx = xcsr(A)
			pixely = vcsr(A)
			if (DimSize(AlphaW, 2) <= 1)
				layer = 0
			else
				NVAR gLayer = root:Packages:WM3DImageSlider:AphaImage:gLayer
				layer = gLayer
			endif
		else
			DoAlert 0, "Please place cursor A on the pixel you want to examine in Intensity or AlphaImage."
			Abort
		endif		
	endif

	// read off parameter values from note attached to Signal
	String s = note(Signal)
	a =	NumberByKey("S", s, "=");				tg = NumberByKey("tg", s, "=");		sigma = NumberByKey("sigma", s, "=");	
	deltat = NumberByKey("deltat", s, "=");		rho = NumberByKey("rho", s, "=");	theta1 = NumberByKey("theta1", s, "=")
	theta2 = NumberByKey("theta2", s, "="); 	b = NumberByKey("b", s, "=");			tau1 = NumberByKey("tau1", s, "=")
	if (Option == 3)
		tau2 = NumberByKey("tau2", s, "=")
	endif
	// make the wave to hold the arrival times in this pixel
	Variable counts = Intensity[pixelx][pixely][layer]
	if (counts == 0)
		DoAlert 0, "No photons in this pixel!"
		Abort
	endif 
	Make/O/N=(counts) pixelTimes		// +2 to hold t=0 and t=1563 points
	pFLIM_SinglePixelInfo /N={pixelx, pixely, layer, counts} /F={StartFrame,EndFrame}  /W=(TimeWindow)  /C=(Channel) /A=pixelTimes /S=fullsorted
	// change from bin number to ns
	pixelTimes *= Binsize
	
	// now create cumulative wave
	Make/O/N=(counts) pixelCum
	pixelCum = (2*p+1)/(2*counts)

	// make cumulative and time waves with 0 and 1 endpoints
	Duplicate/O pixelTimes, pixelTimesL
	Duplicate/O pixelCum, pixelCumL
	InsertPoints 0, 1, pixelTimesL, pixelCumL		// add point at start
	Redimension/N=(counts+2) pixelTimesL, pixelCumL	// add point at end
	pixelTimesL[0] = 0
	pixelCumL[0] = 0
	pixelTimesL[counts+1] = pnt2x(hist,numpnts(hist)-1)
	pixelCumL[counts+1] = 1
	
	// now calculate the model
	Variable k
	Make/D/O/N=(numpnts(hist)+1) Hist1Array, pixelTau1B, Hist2Array, pixelTau2B, pixelB
	SetScale/P x, 0, deltax(hist), Hist1Array, pixelTau1B, Hist2Array, pixelTau2B, pixelB
	Calc3Lifetimes(a, tg, sigma, deltat, rho, theta1, theta2, 0, tau1, tau2, 0, 1, 0, SyncPeriod, Binsize, Hist1Array)	// non-FRET fraction
	pixelTau1B[0] = Hist1Array[0]	// single exp
	for (k = 1; k <= numpnts(hist); k += 1)
		pixelTau1B [k] = Hist1Array[k] + pixelTau1B[k-1]
	endfor
	pixelTau1B[] /= pixelTau1B[numpnts(hist)]		//normalize
	if (Option == 3)
		Calc3Lifetimes(a, tg, sigma, deltat, rho, theta1, theta2, 1, tau1, tau2, 0, 1, 0, SyncPeriod, Binsize, Hist2Array)	// FRET fraction
		pixelTau2B[0] = Hist2Array[0]	// double exp
		for (k = 1; k <= numpnts(hist); k += 1)
			pixelTau2B [k] = Hist2Array[k] + pixelTau2B[k-1]
		endfor
		pixelTau2B[] /= pixelTau2B[numpnts(hist)]		
		//normalize
	endif				
	pixelB = b * p							// background
	pixelB/= pixelB[numpnts(hist)]		//normalize
	pixelB *= (1-Signal[pixelx][pixely]/Intensity[pixelx][pixely])
	pixelTau1B *= Signal[pixelx][pixely]/Intensity[pixelx][pixely]

	if (Option == 3)		// for double-exp lifetimes
		Variable betas = AlphaW[pixelx][pixely]/tau1 / ((1-AlphaW[pixelx][pixely])/tau2 + AlphaW[pixelx][pixely]/tau1)
		pixelTau1B *= (1-betas)
		pixelTau2B *= Signal[pixelx][pixely]/Intensity[pixelx][pixely] * betas						// Total = Signal * ((1-beta)*tau1 + beta*tau2) + (1-Signal)*Background
	endif		
	pixelTau1B += pixelB
	if (Option == 3)
		pixelTau2B += pixelB		// tau2 + Background 
		Duplicate/O pixelTau2B,  pixelTau1Tau2B	
		pixelTau1Tau2B += pixelTau1B - pixelB	// tau1 + tau2 + Background		
		DisplayPixelInfo(counts, Signal[pixelx][pixely], betas, AlphaW[pixelx][pixely])		// display in a new graph
	else
		DisplayPixelInfo(counts, Signal[pixelx][pixely], 0, 0)		// display in a new graph
	endif
	KillWaves/Z param, Hist1Array, Hist2Array, pixelTimes, pixelTimesL, pixelCum, pixelCumL, pixelB, pixelTau1B, pixelTau2B, pixelTau1Tau2B
	DoWindow/F FLIMPanel 
End

//*******************************************************************************************************
// ***************************************    Image ROI    ****************************************************
//*******************************************************************************************************

// MyCreateImageROIPanel *******************************************************
Function MyCreateImageROIPanel(ctrlName) : ButtonControl
	String ctrlName	
	NVAR Colors=root:Data:Colors
	if (Colors == 0)	// blue
		String back = "(5000,24817,52428)"
		String panel = "(56151,63031,65535)"
		String sub = "(49151,60031,65535)"
		String buttons = "(4768,34615,65535)"
	elseif (Colors == 1)
		back = "(4*65535/256,48*65535/256,70*65535/256)"
		panel = "(182*65535/256,244*65535/256,110*65535/256)"
		sub = "(124*65535/256,231*65535/256,0*65535/256)"
		buttons = "(256*65535/256,54*65535/256,0*65535/256)"
	elseif (Colors == 2)
		back = "(0*65535/256,0*65535/256,102*65535/256)"
		panel = "(255*65535/256,255*65535/256,204*65535/256)"
		sub = "(204*65535/256,204*65535/256,255*65535/256)"
		buttons = "(153*65535/256,0*65535/256,102*65535/256)"
	elseif (Colors == 3)
		back = "(0*65535/256,99*65535/256,150*65535/256)"
		panel = "(198*65535/256,240*65535/256,254*65535/256)"
		sub = "(156*65535/256,226*65535/256,252*65535/256)"
		buttons = "(190*65535/256,33*65535/256,50*65535/256)"
	endif
	DoWindow/F MyImageROIPanel
	if( V_Flag==1 )
		return 0
	endif
	String igName= WMTopImageGraph()
	if( strlen(igName) == 0 )
		DoAlert 0,"No image plot found"
		return 0
	endif
	String fonts = ",font =\"Arial\",fSize=13"
	NewPanel /K=1 /W=(300,100,450,280) as "ROI"
	DoWindow/C MyImageROIPanel
	AutoPositionWindow/E/M=1/R=$igName
	Execute ("ModifyPanel fixedSize=1,cbRGB=" +panel)
	Execute "Button StartROI,pos={14,9},size={120,20},proc=MyRoiDrawButtonProc,title=\"Start ROI Draw\",fcolor=" + buttons + fonts 
	Button StartROI,help={"Adds drawing tools to top image graph. Use rectangle, circle or polygon."}
	Execute "Button clearROI,pos={14,63},size={120,20},proc=WMRoiDrawButtonProc,title=\"Erase ROI\",fcolor=" + buttons + fonts
	Button clearROI,help={"Erases previous ROI. Not undoable."}
	Execute "Button FinishROI,pos={14,35},size={120,20},proc=MyRoiDrawButtonProc,title=\"Finish ROI\",fcolor=" + buttons + fonts
	Button FinishROI,help={"Click after you are finished editing the ROI"}
	Execute "Button saveROICopy,pos={14,92},size={120,20},proc=MysaveRoiCopyProc,title=\"Save ROI Copy\",fcolor=" + buttons + fonts
	Execute "CheckBox zeroRoiCheck,pos={15,122},size={94,14},title=\"Zero ROI Pixels\",value= 1" + fonts
	Execute "Button roiPanelHelp,pos={15,145},size={120,20},proc=MyroiPanelButtonProc,title=\"Help\",fcolor=" + buttons + fonts
end

// MySaveROICopyProc *******************************************************
Function MySaveRoiCopyProc(ctrlName) : ButtonControl
	String ctrlName	
	NVAR Show = root:Data:Show
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	String topWave=WMGetImageWave(WMTopImageGraph())
	WAVE/Z ww=$topWave
	if(WaveExists(ww)==0)
		return 0
	endif
	
	String waveDF=GetWavesDataFolder(ww,1 )
	SetDataFolder root:Analysis:ROIMasks
	Wave/T ROIMaskTable, ROIMaskList
	String Mask
	Variable i=0
	do 
		i += 1
		Mask = ROIMaskTable[i][Show]
	while (strlen(Mask) > 1)

	ControlInfo zeroRoiCheck
	if(V_value)
		ImageGenerateROIMask/E=1/I=0 $WMTopImageName()		
	else
		ImageGenerateROIMask $WMTopImageName()		
	endif
	
	Wave M_ROIMask
	String ROIName = "ROIMask" + num2str(i) + "_" + num2str(Show)
	Duplicate M_ROIMask $ROIName
	ROIMaskTable[i][Show] = ROIName
	ROIMaskList = ROIMaskTable[p][Show]
	DoWindow/F FLIMpanel
	ListBox listROIs, selRow=i
	ControlUpdate/W=FLIMpanel listROIs	
End

// MyROIDrawButtonProc *******************************************************
Function MyRoiDrawButtonProc(ctrlName) : ButtonControl
	String ctrlName	
	String ImGrfName= WMTopImageGraph()
	if( strlen(ImGrfName) == 0 )
		return 0
	endif
	
	DoWindow/F $ImGrfName
	if( CmpStr(ctrlName,"StartROI") == 0 )
		ShowTools/A rect
		SetDrawLayer ProgFront
		Wave w= $WMGetImageWave(ImGrfName)		// the target matrix
		String iminfo= ImageInfo(ImGrfName, NameOfWave(w), 0)
		String xax= StringByKey("XAXIS",iminfo)
		String yax= StringByKey("YAXIS",iminfo)
		SetDrawEnv linefgc= (3,52428,1),fillpat= 0,xcoord=$xax,ycoord=$yax,save
	endif
	if( CmpStr(ctrlName,"FinishROI") == 0 )
		GraphNormal
		HideTools/A
		SetDrawLayer UserFront
		DoWindow/F MyImageROIPanel
	endif
	if( CmpStr(ctrlName,"clearROI") == 0 )
		GraphNormal
		SetDrawLayer/K ProgFront
		SetDrawLayer UserFront
		DoWindow/F MyImageROIPanel
	endif
End
	
// MyROIPanelButtonProc *******************************************************	
Function MyRoiPanelButtonProc(ctrlName) : ButtonControl
	String ctrlName
	DisplayHelpTopic "ROI Panel"
End

// MyROIUpdateProc *******************************************************
Function MyROIUpdateProc(ctrlName,row,col,event) : ListboxControl
	String ctrlName     // name of this control
	Variable row        // row if click in interior, -1 if click in title
	Variable col        // column number
	Variable event      // event code
	
	if (row >= 0 && event == 4)	// execute only if row is > -1 and a row was selected by mouse or keyboard
//		DoWindow/F IntensityImage		// if the Window is not there, display it
//		if (V_Flag==0)
//			NVAR Flag = root:Data:IntensityFlag
//			Flag = 1
//			DisplayIntensity("",0)
//		endif
//		// now that there is a window, apply the mask to it
//		if (row == 0)
//			DisplayIntensity("",0)
//		else
//			SetDataFolder root:Analysis:ROIMasks
//			Wave/T MaskList = ROIMaskList
//			String Mask = MaskList[row]
//			String WaveNameList = ImageNameList("IntensityImage", "") 
//			Variable i = 1
//			String Name = StringFromList(i, WaveNameList)
//			if (strlen(Name) > 0)
//				do
//					RemoveImage/Z/W=IntensityImage $Name
//					i += 1
//					Name = StringFromList(i, WaveNameList)
//				while (strlen(Name) > 0)
//			endif
//			if (WaveExists($Mask))
//				AppendImage/G=1 $Mask
//				ModifyImage $Mask ctab= {0.01,1,Grays,0}, minRGB=NaN,maxRGB=1
//			endif
//		endif
		DisplayIntensity("",1)
		DisplayPixelAnalysisResults("",1)
	endif
	return 0            // other return values reserved
End

//*********************************************
Function CopyROI(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Analysis:Pixels
	NVAR Show = root:Data:Show
	NVAR IntensityFlag
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	Variable Binning = FileInfo_Variables[Show][%Binning]
	String IntensityName = "root:Analysis:Pixels:Intensity_B" + num2str(Binning) + "_"  + num2str(Show) + "_Bin"	// this ending is automatically added by Igor when saving the threshold image
	// now the region of interest has value 255, outside 0
	// what we want is ROI values of 0, outside 1
	Wave Int = $IntensityName	
	Int = -1*(Int/255 - 1)
	SetDataFolder root:Analysis:ROIMasks	// now copy the image to the right place under the right name
	Wave/T ROIMaskTable, ROIMaskList
	String Mask
	Variable i=0
	do 
		i += 1
		Mask = ROIMaskTable[i][Show]
	while (strlen(Mask) > 1)
	Wave M_ROIMask
	String ROIName = "ROIMask" + num2str(i) + "_" + num2str(Show)
	Duplicate $IntensityName $ROIName
	ROIMaskTable[i][Show] = ROIName
	ROIMaskList = ROIMaskTable[p][Show]
	DoWindow/F FLIMpanel
	ListBox listROIs, selRow=i
	ControlUpdate/W=FLIMpanel listROIs	
	KillWaves/Z $IntensityName
End

//********************************************************************************************//
//********************************************** Display Functions *****************************//
//********************************************************************************************//

// DisplayTimeHist *********************************
Function DisplayTimeHist(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked			// 1 if selelcted, 0 if not
	SetDataFolder root:Data
	NVAR Show, TimeHistFlag
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	String Histname, Appendname, Gaussname, C1name, C2name, Resname, Tau1Name, Tau2Name, Tau3Name
	Variable line=1.5
	Histname = "hist_" + num2str(Show)		
	Make/O/T Names={"", "IRF", "Single", "Double","Triple"}
	DoWindow/F TimeHist
	if (TimeHistFlag == 1 && WaveExists($Histname))
		if(V_Flag==1)	// if there is a window, remove image on it
			String OldWaveNameList = TraceNameList("", ",",1) 
			String OldWaveName = StringFromList(0, OldWaveNameList, ",")
			ReplaceWave/W=TimeHist trace=$OldWaveName $Histname
			// Now remove all other traces from Graph
			Variable items = ItemsInList(OldWaveNameList, ","), i
			for (i = 1; i < items; i+=1)
				OldWaveName = StringFromList(i, OldWaveNameList, ",")
				RemoveFromGraph/W=TimeHist $OldWaveName
			endfor
			ModifyGraph axisEnab(left)={0.2,0.75}, notation(left)=1, lsize($Histname)=2,rgb($Histname)=(0,0,0), lblPos(left)=65, lblMargin(bottom)=3, axOffset(left)=-2
		else
			Display/W=(620,0,1570,300)
			DoWindow/C TimeHist
			AppendToGraph $Histname
			Label bottom "Time (ns)"		
			Label left "TCSPC"
			ModifyGraph axisEnab(left)={0.2,0.75}, rgb($Histname)=(0,0,0), notation(left)=1, lsize($Histname)=2, lblPos(left)=65, lblMargin(bottom)=3, axOffset(left)=-2
			ShowInfo		
		endif
		// Check for Tau1- and Tau2Waves
		if (Option == 3 || Option == 4)	// double or triple exp
			SetDataFolder $("root:Analysis:Manual:" + Names[Option])
			Tau1name = "Tau1_" + Names[Option] +num2str(Show)
			Tau2name = "Tau2_" + Names[Option] +num2str(Show)
			AppendToGraph/W=TimeHist/L=left  $Tau1name, $Tau2name
			ModifyGraph rgb($Tau1name)=(0,34695,52428), rgb($Tau2name)=(65535,32639,0), lsize($Tau1name)=line, lsize($Tau2name)=line
		endif
		// Check for Tau3Wave
		if (Option == 4)	// triple exp
			SetDataFolder $("root:Analysis:Manual:" + Names[Option])
			Tau3name = "Tau3_" + Names[Option] +num2str(Show)
			AppendToGraph/W=TimeHist/L=left  $Tau3name
			ModifyGraph rgb($Tau3name)=(36873,14755,58982), lsize($Tau3name)=line
		endif
		// add automatic fits 	
		SetDataFolder $("root:Analysis:Fits:" + Names[Option])
		Appendname = Names[Option] + "Fit_"+num2str(Show)
		if (WaveExists($Appendname ))
			AppendToGraph/W=TimeHist $Appendname
			ModifyGraph lsize($Appendname)=line,rgb($Appendname)=(65535,43690,0)
		endif
		// now add the manual fits
		SetDataFolder $("root:Analysis:Manual:" + Names[Option])
		Appendname = Names[Option] + "Init_"+num2str(Show)
		if (WaveExists($Appendname ))
			AppendToGraph/W=TimeHist $Appendname
			ModifyGraph lsize($Appendname)=line,rgb($Appendname)=(65535,0,0)
		endif	
		// Check for existing residuals
		SetDataFolder $("root:Analysis:Fits:" + Names[Option])
		Resname = "Res_" + Names[Option] + "Fit_"+num2str(Show)
		if (WaveExists($Resname))
			AppendToGraph/W=TimeHist $Resname
			ModifyGraph rgb($Resname)=(2,39321,1)
			// now append normalized residuals on new left axis
			Resname = "NormRes_" + Names[Option] + "Fit_"+num2str(Show)
			AppendToGraph/W=TimeHist/L=Res_left $Resname
			ModifyGraph rgb($Resname)=(2,39321,1), nticks(Res_left)=2
			Label Res_left "Normalized\rResiduals"
			ModifyGraph axisEnab(Res_Left)={0,0.15}, lblPos(Res_Left)=65, freePos(Res_left)=0
		endif
		// Check for existing components
		SetDataFolder $("root:Analysis:Manual:" + Names[Option])
		Gaussname = "Gaussian_" + Names[Option] +num2str(Show)
		C1name = "Comp1_" + Names[Option] +num2str(Show)
		if (WaveExists($Gaussname))			
			AppendToGraph/W=TimeHist/L=Comp_left $Gaussname, $C1name
			ModifyGraph rgb($Gaussname)=(1,12815,52428), lsize($Gaussname)=line, rgb($C1name)=(52428,1,41942), lsize($C1name)=line
			ModifyGraph axisEnab(Comp_Left)={0.8,1}, lblPos(Comp_Left)=65, freePos(Comp_left)=0, nticks(Comp_left)=2
			Label Comp_left "IRF Model"
			if (Option == 3)
				Legend/C/A=RT/N=text0/J/F=0/B=1 "\\s(" + Gaussname + ") Gaussian component\r\\s("+ C1name +") Exp. component\r\r\\s(" + Histname + ") TCSPC\r\\s(" + Appendname + ") Fit\r\\s(" + Resname + ") Residuals\r\\s("+ Tau1name + ") \F'Symbol't\F'Arial'\B1\M + Background\r\\s("+ Tau2name +") \F'Symbol't\F'Arial'\B2\M + Background"
			elseif (Option == 4)
				Legend/C/A=RT/N=text0/J/F=0/B=1 "\\s(" + Gaussname + ") Gaussian component\r\\s("+ C1name +") Exp. component\r\r\\s(" + Histname + ") TCSPC\r\\s(" + Appendname + ") Fit\r\\s(" + Resname + ") Residuals\r\\s("+ Tau1name + ") \F'Symbol't\F'Arial'\B1\M + Background\r\\s("+ Tau2name +") \F'Symbol't\F'Arial'\B2\M + Background\r\\s("+ Tau3name +") \F'Symbol't\F'Arial'\B3\M + Background"
			else
				Legend/C/A=RT/N=text0/J/F=0/B=1 "\\s(" + Gaussname + ") Gaussian component\r\\s("+ C1name +") Exp. component\r\r\\s(" + Histname + ") TCSPC\r\\s(" + Appendname + ") Fit\r\\s(" + Resname + ") Residuals"
			endif
		endif
	else
		DoWindow/K TimeHist	// kill window
	endif
	KillWaves/Z Names
End

// DisplayIntensity *********************************
Function DisplayIntensity(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked			// 1 if selelcted, 0 if not
	SetDataFolder root:Analysis:Pixels	
	NVAR Show = root:Data:Show
	NVAR IntensityFlag
	SVAR ModGraph, Color1, Color2
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	Variable Binning = FileInfo_Variables[Show][%Binning]
	String IntensityName = "Intensity_B" + num2str(Binning) + "_"  + num2str(Show)			
	DoWindow/K/Z IntensityImage
	if (IntensityFlag == 1 && WaveExists($IntensityName))
		Display/W=(500,400,750,602)
		DoWindow/C IntensityImage
		AppendImage/G=1/L=left/T=top $IntensityName
		Execute(ModGraph)	
		ModifyImage $IntensityName ctab= {*,*,Red,0}
		Execute(Color1 + IntensityName + Color2)
		if (DimSize($IntensityName, 2) > 1)
			WMAppend3DImageSlider()
		endif
		// check if we want to apply an ROI
		ControlInfo/W=FLIMpanel listROIs
		 if (V_value >= 0)
			Wave/T MaskList = root:Analysis:ROIMasks:ROIMaskList
			String Mask = MaskList[V_value]
			Wave MaskUsed = $( "root:Analysis:ROIMasks:" + Mask)
			if (WaveExists(MaskUsed))
				AppendImage/G=1 MaskUsed
				ModifyImage $Mask ctab= {0.01,1,Grays,0}, minRGB=NaN,maxRGB=1
			endif
		 endif
		ShowInfo
	endif
End

// DisplayPixelAnalysisResults *********************************
Function DisplayPixelAnalysisResults(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked			// 1 if selelcted, 0 if not
	NVAR Show = root:Data:Show
	SetDataFolder root:Analysis:Pixels
	NVAR SignalFlag, BackgroundFlag, AlphaFlag, BetaFlag, AvgTauFlag, DisplayLowerTau, DisplayUpperTau, DisplayLowerAlpha, DisplayUpperAlpha
	SVAR ModGraph, Color1, Color2
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	Variable Binning = FileInfo_Variables[Show][%Binning]
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	
	SetDataFolder root:Analysis:Pixels:	
	String SName = "Signal_B" + num2str(Binning ) + "_" + num2str(Show)
	String Back = "Background_B" + num2str(Binning ) + "_" + num2str(Show)
	
	DoWindow/K/Z SignalImage
	DoWindow/K/Z BackgroundImage
	DoWindow/K/Z BetaImage
	DoWindow/K/Z AlphaImage
	DoWindow/K/Z AvgTauImage
	
	if (SignalFlag == 1 && WaveExists($SName))
		Display/W=(750,400,1000,602)
		DoWindow/C SignalImage
		AppendImage/G=1/L=left/T=top $SName
		Execute(ModGraph)
		ModifyImage $SName  ctab= {*,*,Red,0}
		Execute(Color1 + SName + Color2)
		if (DimSize($SName, 2) > 1)
			WMAppend3DImageSlider()
		endif
	endif
	
	if (BackgroundFlag == 1 && WaveExists($Back))
		Display/W=(650,400,900,602)
		DoWindow/C BackgroundImage
		AppendImage/G=1/L=left/T=top $Back
		Execute(ModGraph)
		ModifyImage $Back ctab= {*,*,YellowHot,0}
		Execute(Color1 + Back + Color2)
		if (DimSize($SName, 2) > 1)
			WMAppend3DImageSlider()
		endif
		Wave Hist = $("root:Data:hist_" + num2str(Show))
		TextBox/C/N=text1/F=0/A=LT "Total/Bin = " + num2str(sum($Back,Inf,-Inf)/numpnts(Hist))
	endif
	
	if (Option == 3)	// change the rest of the double-exp images	
	// in case an ROI Mask is selected, apply it to the images
		ControlInfo/W=FLIMpanel listROIs	
		if (V_Value > 0)
			Wave/T MaskList = root:Analysis:ROIMasks:ROIMaskList
			String Mask = MaskList[V_value]
			Wave MaskUsed = $("root:Analysis:ROIMasks:" + Mask)
		endif
	
		String BetaName = "Beta_B" + num2str(Binning ) + "_" + num2str(Show)
		String AlphaName = "Alpha_B" + num2str(Binning ) + "_" + num2str(Show)
		String AvgName = "AvgTau_B" + num2str(Binning ) + "_" + num2str(Show)	
		DoWindow/F BetaImage
		if (BetaFlag == 1 && WaveExists($BetaName))
			Display/W=(0,625,250,827)
			DoWindow/C BetaImage
			AppendImage/G=1/L=left/T=top $BetaName
			Execute(ModGraph)
			ModifyImage $BetaName ctab= {0,1,Rainbow,0}
			Execute(Color1 + BetaName + Color2)
			if (DimSize($BetaName, 2) > 1)
				WMAppend3DImageSlider()
			endif
			if (V_Value > 0)	// if a mask is selected
				AppendImage/W=BetaImage MaskUsed
				ModifyImage $Mask ctab= {0.01,0.9,Grays,0}, minRGB=NaN,maxRGB=(0,0,0)
			endif
		endif
		if (AlphaFlag == 1 && WaveExists($AlphaName))
			Display/W=(0,625,250,827)
			DoWindow/C AlphaImage
			AppendImage/G=1/L=left/T=top $AlphaName
			Execute(ModGraph)		
			ModifyImage $AlphaName ctab= {DisplayLowerAlpha,DisplayUpperAlpha,Rainbow,0}
			Execute(Color1 + AlphaName + Color2)
			if (DimSize($AlphaName, 2) > 1)
				WMAppend3DImageSlider()
			endif
			if (V_Value > 0)	// if a mask is selected
				AppendImage/W=AlphaImage  MaskUsed
				ModifyImage $Mask ctab= {0.01,0.9,Grays,0}, minRGB=NaN,maxRGB=(0,0,0)
			endif
			ShowInfo
		endif
		if (AvgTauFlag == 1 && WaveExists($AvgName))
			Display/W=(500,625,750,827)
			DoWindow/C AvgTauImage
			AppendImage/G=1/L=left/T=top $AvgName
			Execute(ModGraph)
			ModifyImage $AvgName ctab= {DisplayLowerTau,DisplayUpperTau,Rainbow,1}
			Execute(Color1 + AvgName + Color2)
			if (DimSize($AvgName, 2) > 1)
				WMAppend3DImageSlider()
			endif
			if (V_Value > 0)	// if a mask is selected
				AppendImage/W=AvgTauImage  MaskUsed
				ModifyImage $Mask ctab= {0.01,0.9,Grays,0}, minRGB=NaN,maxRGB=(0,0,0)
			endif
			
		endif
	endif
End

// DisplayPixelInfo *********************************
Function DisplayPixelInfo(counts, s, betas, alpha)
	Variable  counts, s, betas, alpha
	NVAR Show = root:Data:Show
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	SetDataFolder root:Analysis:Pixels
	Wave pixelTimes, pixelTimesL, pixelCum, pixelCumL, pixelB, pixelTau1B
	if (Option == 3)
		Wave pixelTau1Tau2B, pixelTau2B
	endif
	
	DoWindow/F PixelInfo
	if(V_Flag==1)	// if there is a window, kill it
		DoWindow/K PixelInfo
	endif
	Display/K=1/W=(620,0,1070,250)
	DoWindow/C PixelInfo	
	AppendToGraph/W=PixelInfo pixelCumL vs pixelTimesL
	ModifyGraph mode(pixelCumL)=0, rgb(pixelCumL)=(0,0,0)
	AppendToGraph/W=PixelInfo pixelCum vs pixelTimes
	ModifyGraph mode(pixelCum)=4, marker(pixelCum)=19, rgb(pixelCum)=(0,0,0)
	Label bottom "Time (ns)"		
	Label left "Cumulative"
	AppendToGraph/W=PixelInfo pixelB , pixelTau1B
	ModifyGraph rgb(pixelB)=(39321,1,31457),rgb(pixelTau1B)=(0,34695,52428)
	ModifyGraph lsize(pixelTau1B)=1.5, lsize(pixelB)=1.5
	if (Option == 3)
		AppendToGraph/W=PixelInfo  pixelTau1Tau2B,  pixelTau2B
		ModifyGraph rgb(pixelTau1Tau2B)=(65535,0,0), lsize(pixelTau1Tau2B)=1.5
		ModifyGraph rgb(pixelTau2B) = 	(65535,32639,0), lsize(pixelTau2B)=1.5
		Legend/C/N=text0/A=RC/J "\\Z12N = " + num2str(counts) 
		AppendText/N=text0		"\Z12S =" + num2str(s)
		AppendText/N=text0		"\Z12\\F'Symbol'b\\F'Arial' = " + num2str(betas)
		AppendText/N=text0		"\Z12\\F'Symbol'a\\F'Arial' = " + num2str(alpha)
		AppendText/N=text0		"\Z12\\s(pixelCum) Photons"
		AppendText/N=text0		"\Z12\\s(pixelTau1Tau2B) Fit" 
		AppendText/N=text0 		"\Z12\\s(pixelTau1B) \\F'Symbol't\\B1\\M\Z12 + \\F'Arial'b"
		AppendText/N=text0 		"\Z12\\s(pixelTau2B) \\F'Symbol't\\B2\\M\Z12 + \\F'Arial'b"
		AppendText/N=text0 		"\Z12\\s(pixelB) b"
	else
		Legend/C/N=text0/A=RC/J "\\Z12N = " + num2str(counts)
		AppendText/N=text0		"\Z12\\s(pixelCum) Pixel" 
		AppendText/N=text0 		"\Z12\\s(pixelTau1B) \\F'Symbol' t\\B2\\M\Z12 + \\F'Arial'b"
		AppendText/N=text0		"\Z12\\s(pixelB) b"
	endif
End

// DisplayWeighted *********************************
Function DisplayWeighted(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked			// 1 if selelcted, 0 if not
	NVAR Show = root:Data:Show
	SetDataFolder root:Analysis:Pixels
	NVAR AlphaWeightedFlag, BetaWeightedFlag, AvgTauWeightedFlag, DisplayLowerTau, DisplayUpperTau, DisplayLowerAlpha, DisplayUpperAlpha
	SVAR ModGraph, ColorW, Color2
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	Variable Binning = FileInfo_Variables[Show][%Binning]
	// first kill any windows that might be there
	DoWindow/K/Z BetaWeighted
	DoWindow/K/Z AlphaWeighted
	DoWindow/K/Z AvgTauWeighted

	if (Option == 3)	// continue with the rest of the double-exp windows
		String BetaNameW = "Beta_B" + num2str(Binning ) + "_W_" + num2str(Show)
		if (BetaWeightedFlag == 1 && WaveExists($BetaNameW))
			Display/W=(250,625,500,827)
			DoWindow/C BetaWeighted
			AppendImage/L=left/T=top $BetaNameW
			Execute(ModGraph)
			Variable ReverseColor=0		// don't reverse
			Execute (ColorW + "{0,1,Rainbow,"+num2str(ReverseColor)+"}" + Color2)
			if (DimSize($BetaNameW, 3) > 1)
				RGBAppend3DImageSlider()
			endif
		endif
		String AlphaNameW = "Alpha_B" + num2str(Binning ) + "_W_" + num2str(Show)
		if (AlphaWeightedFlag == 1 && WaveExists($AlphaNameW))
			Display/W=(250,625,500,827)
			DoWindow/C AlphaWeighted
			AppendImage/L=left/T=top $AlphaNameW
			Execute(ModGraph)
			ReverseColor=0		// don't reverse
			Execute (ColorW + "{"+num2str(DisplayLowerAlpha)+","+num2str(DisplayUpperAlpha)+",Rainbow,"+num2str(ReverseColor)+"}" + Color2)
			if (DimSize($AlphaNameW, 3) > 1)
				RGBAppend3DImageSlider()
			endif
		endif
		String AvgNameW = "AvgTau_B" + num2str(Binning ) + "_W_" + num2str(Show)
		if (AvgTauWeightedFlag == 1 && WaveExists($AvgNameW))
			Display/W=(750,625,1000,827)
			DoWindow/C AvgTauWeighted
			AppendImage/L=left/T=top $AvgNameW
			Execute(ModGraph)
			ReverseColor=1		// reverse
			Execute (ColorW + "{"+num2str(DisplayLowerTau)+","+num2str(DisplayUpperTau)+",Rainbow,"+num2str(ReverseColor)+"}" + Color2)
			if (DimSize($AvgNameW, 3) > 1)
				RGBAppend3DImageSlider()
			endif
		endif
	endif
	DoUpdate
End

//*****************************************************************************************//
// PrettyLayout **********************************************************************
Function CreateBackgroundImage()
	SetDataFolder root:Data
	NVAR Show
	Wave FileInfo_Variables
	SetDataFolder root:Analysis:Pixels
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Binning = FileInfo_Variables[Show][%Binning]	
	// load all relevant waves here into the function so we can access them
	Wave Intensity = $("Intensity_B" + num2str(Binning) + "_"  + num2str(Show))
	Wave Signal = $("Signal_B" + num2str(Binning) + "_" + num2str(Show))
	String BackName = "Background_B" + num2str(Binning ) + "_" + num2str(Show)
	Duplicate/O Intensity, $BackName
	Wave Back = $BackName
	Back -=  Signal
	DisplayPixelAnalysisResults("",1)	// Show Images
End

Function CreateBetaImage()
	SetDataFolder root:Data
	NVAR Show
	Wave FileInfo_Variables
	SetDataFolder root:Analysis:Pixels
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Binning = FileInfo_Variables[Show][%Binning]	
	Variable xSize = FileInfo_Variables[Show][%xSize] / Binning
	Variable ySize = FileInfo_Variables[Show][%ySize] / Binning
	// load all relevant waves here into the function so we can access them
	Wave Signal = $("Signal_B" + num2str(Binning) + "_" + num2str(Show))
	Wave Alpha = $("Alpha_B" + num2str(Binning ) + "_" + num2str(Show))
	String BetaName = "Beta_B" + num2str(Binning) + "_" + num2str(Show)
	String BetaNameW = "Beta_B" + num2str(Binning ) + "_W_" + num2str(Show)
	// get the note attached to Signal to obtain the tau1 and tau2 values used to create the images
	String Info = note(Signal)
	Variable tau1 = NumberByKey("tau1", Info, "=", ";")
	Variable tau2 = NumberByKey("tau2", Info, "=", ";")
	Duplicate/O Signal, $BetaName
	Wave Betas = $BetaName
	Betas = Alpha/tau1 / ((1-Alpha)/tau2 + Alpha/tau1)
	// now create the weighted Beta image
	Variable layers = DimSize(Signal, 2)
	if (layers == 0)
		layers = 1
	endif
	// first convert the grey image to a 3-layer RGB image using the Rainbow color table
	ColorTab2Wave Rainbow		// extracts colors from the built-in color table and places them in an Nx3 matrix of red, green and blue columns 
	Wave M_colors
	SetScale/I x, 0, 1, M_colors
	Make/O/N=(xSize, ySize, 3) temp										// make image with 3 layers for RGB
	Make/O/N=(xSize, ySize, 3, layers ) $BetaNameW		// 3 layers for RGB, the 4th dimension contains the frames
	Wave BetaW = $BetaNameW
	Variable h, i, k
	for (h = 0; h < layers ; h += 1)	// go through each frame
		for(i=0; i<xSize; i+= 1)		// go thru rows
			for(k=0; k<ySize; k += 1)	// go thru columns
				temp[i][k][] = M_colors[x2pnt(M_colors, Betas[i][k][h])][r]
			endfor
		endfor
		ImageTransform/U rgb2hsl temp		// now convert this RGB image to an HSL image (hue, saturation, light)
		Wave M_RGB2HSL						// result of the transformation	
		WaveStats/Q/M=1 Signal
		M_RGB2HSL[][][2] = Signal[p][q][h]/V_max * 65535		// now replace the light layer with the Signal image and normalize
		ImageTransform hsl2rgb M_RGB2HSL		// finally convert this back to RGB
		Wave M_HSL2RGB							// result of the transformation
		BetaW[][][][h] = M_HSL2RGB[p][q][r]
	endfor
	DisplayPixelAnalysisResults("",1)	// Show Images
	KillWaves/Z temp, M_RGB2HSL, M_HSL2RGB, M_colors
End

Function PrettyLayout()
	NVAR Show = root:Data:Show
	DoWindow/F IntensityImage
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	DoWindow/F BackgroundImage
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	DoWindow/F SignalImage
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	DoWindow/F BetaImage
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	DoWindow/F AlphaImage
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	DoWindow/F AvgTauImage
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	DoWindow/F BetaWeighted
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	DoWindow/F AlphaWeighted
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	DoWindow/F AvgTauWeighted
	ColorScale/C/N=text0/X=-34.00/Y=5.00
	NewLayout/K=1/W=(10,10,900,670)/P=landscape
	ModifyLayout mag=1
	Variable o=60, m=240, u=425
	Variable l=20, ml=215, mr=410, r=605
	Variable width=195, height=147
	Variable diffo=15
	AppendLayoutObject/R=(l,	o,	l+width,	o+height) graph IntensityImage
	AppendLayoutObject/R=(ml,o,	ml+width,	o+height) graph BackgroundImage
	AppendLayoutObject/R=(mr,o,	mr+width,	o+height) graph SignalImage
	AppendLayoutObject/R=(l, m,	l+width,	m+height) graph BetaImage
	AppendLayoutObject/R=(ml,m,	ml+width,	m+height) graph AlphaImage
	AppendLayoutObject/R=(mr,m,	mr+width,	m+height) graph AvgTauImage
	AppendLayoutObject/R=(l,u,	l+width,	u+height) graph BetaWeighted
	AppendLayoutObject/R=(ml,u,	ml+width,	u+height) graph AlphaWeighted
	AppendLayoutObject/R=(mr,u,	mr+width,	u+height) graph AvgTauWeighted
	Textbox/A=LT "Experiment 30 July 2009\rMDCK cells transfected with\rcaspase sensor\rStaurosporin\rLSM_ "+num2str(Show+1)
	ModifyLayout left(text0)=615,top(text0)=250
	Textbox/F=0/A=LT "Intensity Image"
	ModifyLayout left(text1)=(l+10),top(text1)=(o-diffo)
	Textbox/F=0/A=LT "Background counts"
	ModifyLayout left(text2)=(ml+10),top(text2)=(o-diffo)
	Textbox/F=0/A=LT " Signal counts"
	ModifyLayout left(text3)=(mr+10),top(text3)=(o-diffo)
	
	Textbox/F=0/A=LT "\\F'Symbol'b\\F'Arial' values"
	ModifyLayout left(text5)=(l+10),top(text5)=(m-diffo)
	Textbox/F=0/A=LT "\\F'Symbol'a\\F'Arial' values"
	ModifyLayout left(text6)=(ml+10),top(text6)=(m-diffo)
	Textbox/F=0/A=LT "Average \\F'Symbol't\\F'Arial' (ns)"
	ModifyLayout left(text7)=(mr+10),top(text7)=(m-diffo)
	Textbox/F=0/A=LT "Intensity Weighted"
	ModifyLayout left(text8)=(ml+20),top(text8)=(u-20)
End

// DisplayOptions *********************************
Function DisplayOptions()
	NVAR Colors = root:Data:Colors
	if (Colors == 0)	// blue
		String back = "(5000,24817,52428)"
		String panel = "(56151,63031,65535)"
		String sub = "(49151,60031,65535)"
		String buttons = "(4768,34615,65535)"
		String line = "(65535,49151,55704)"
		String chi = "(65535,65533,32768)"
	elseif (Colors == 1)
		back = "(4*65535/256,48*65535/256,70*65535/256)"
		panel = "(182*65535/256,244*65535/256,110*65535/256)"
		sub = "(124*65535/256,231*65535/256,0*65535/256)"
		buttons = "(256*65535/256,54*65535/256,0*65535/256)"
		line = "(256*65535/256,105*65535/256,65*65535/256)"
		chi = "(100*65535/256,174*65533/256,208*65533/256)"
	elseif (Colors == 2)
		back = "(0*65535/256,0*65535/256,102*65535/256)"
		panel = "(255*65535/256,255*65535/256,204*65535/256)"
		sub = "(204*65535/256,204*65535/256,255*65535/256)"
		buttons = "(153*65535/256,0*65535/256,102*65535/256)"
		line = "(153*65535/256,0*65535/256,102*65535/256)"
		chi = "(255*65535/256,255*65535/256,204*65535/256)"
	elseif (Colors == 3)
		back = "(0*65535/256,99*65535/256,150*65535/256)"
		panel = "(198*65535/256,240*65535/256,254*65535/256)"
		sub = "(156*65535/256,226*65535/256,252*65535/256)"
		buttons = "(190*65535/256,33*65535/256,50*65535/256)"
		line = "(255*65535/256,51*65535/256,77*65535/256)"
		chi = "(256*65535/256,256*65535/256,256*65535/256)"
	endif
	String fonts = ",font=\"Arial\",fsize=13"
	String size = ",size={120,15}"
	PauseUpdate; Silent 1		// building window...
	DoWindow/F/K DisplayOptionsPanel
	NewPanel/K=1/W=(300,100,650,295)
	DoWindow/C DisplayOptionsPanel
	Execute("ModifyPanel cbRGB=" + panel)
	Execute "CheckBox checkTimeHist,pos={10,10},title=\"TimeHist Graph\",variable= root:Data:TimeHistFlag,proc=DisplayTimeHist" + size + fonts
	Execute "CheckBox checkIntensity,pos={10,30},title=\"Intensity Image\",variable= root:Analysis:Pixels:IntensityFlag,proc=DisplayIntensity" + size + fonts
	Execute "CheckBox checkSignal,pos={180,10},title=\"Signal Image\",variable= root:Analysis:Pixels:SignalFlag,proc=DisplayPixelAnalysisResults" + size + fonts
	Execute "CheckBox checkBackground,pos={180,30},title=\"Background Image\",variable= root:Analysis:Pixels:BackgroundFlag,proc=DisplayPixelAnalysisResults" + size + fonts
	Execute "CheckBox checkBeta,pos={10,50},title=\"\F'Symbol'b\F'Arial' Image\",variable= root:Analysis:Pixels:BetaFlag,proc=DisplayPixelAnalysisResults" + size + fonts
	Execute "CheckBox checkAlpha,pos={10,70},title=\"\F'Symbol'a\F'Arial' Image\",variable= root:Analysis:Pixels:AlphaFlag,proc=DisplayPixelAnalysisResults" + size + fonts
	Execute "CheckBox checkAvgTau,pos={10,90},title=\" \F'Symbol'<t>\Bb\M\F'Arial' Image\",variable= root:Analysis:Pixels:AvgTauFlag,proc=DisplayPixelAnalysisResults " + size + fonts
	Execute "CheckBox checkBetaW,pos={180,50},title=\"Weighted \F'Symbol'b\F'Arial' Image\",variable= root:Analysis:Pixels:BetaWeightedFlag,proc=DisplayWeighted" + size + fonts
	Execute "CheckBox checkAlphaW,pos={180,70},title=\"Weighted \F'Symbol'a\F'Arial' Image\",variable= root:Analysis:Pixels:AlphaWeightedFlag,proc=DisplayWeighted" + size + fonts
	Execute "CheckBox checkAvgTauW,pos={180,90},title=\"Weighted \F'Symbol'<t>\Bb\M\F'Arial' Image\",variable= root:Analysis:Pixels:AvgTauWeightedFlag,proc=DisplayWeighted" + size + fonts
	Execute "SetVariable setvarLTau,pos={10,115},size={220,20},title=\"\Z12Limits for \F'Symbol'<t>\Bb\M\Z12\F'Arial' image\",variable= root:Analysis:Pixels:DisplayLowerTau,proc=ChangeRange" + fonts
	Execute "SetVariable setvarUTau,pos={240,115},size={100,20},title=\" \",variable= root:Analysis:Pixels:DisplayUpperTau,proc=ChangeRange" + fonts
	Execute "SetVariable setvarLalpha,	pos={10,140},size={220,20},title=\"\Z12Limits for \F'Symbol'a\F'Arial' Image\",variable= root:Analysis:Pixels:DisplayLowerAlpha,proc=ChangeRange" + fonts
	Execute "SetVariable setvarUalpha,	pos={240,140},size={100,20},title=\" \",variable= root:Analysis:Pixels:DisplayUpperAlpha,proc=ChangeRange" + fonts
	DoUpdate
End


Function GetBestFitS(ctrlName) : ButtonControl
	String ctrlName	
	Variable lastchi, thischi=5000
	NVAR show = root:Data:Show
	Wave Fit = root:Analysis:Fits:FitValuesSingle
	do
		FitTimeHist("buttonSFit")
		ApplyResults ("buttonSApply")
		lastchi = thischi
		thischi = Fit[Show][%Chisq]
	while ((lastchi-thischi) > 0.00001)		
End

Function GetBestFitD(ctrlName) : ButtonControl
	String ctrlName	
	Variable lastchi, thischi=5000
	NVAR show = root:Data:Show
	Wave Fit = root:Analysis:Fits:FitValuesDouble
	do
		FitTimeHist("buttonDFit")
		ApplyResults ("buttonDApply")
		lastchi = thischi
		thischi = Fit[Show][%Chisq]
	while ((lastchi-thischi) > 0.00001)		
End

Function GetBestFitT(ctrlName) : ButtonControl
	String ctrlName	
	Variable lastchi, thischi=5000
	NVAR show = root:Data:Show
	Wave Fit = root:Analysis:Fits:FitValuesTriple
	do
		FitTimeHist("buttonTFit")
		ApplyResults ("buttonTApply")
		lastchi = thischi
		thischi = Fit[Show][%Chisq]
	while ((lastchi-thischi) > 0.00001)		
End


//*****************************************************************************************//
Function SetColorScheme(option)
	Variable option
	NVAR Colors = root:Data:Colors
	Colors = option
	AnalysisPanel()
End

// GlobalFitHistogramsPanel() *******************************************
Function GlobalFitHistogramsPanel()
	NVAR Show = root:Data:Show
	NVAR Colors = root:Data:Colors
	if (Colors == 0)	// blue
		String back = "(5000,24817,52428)"
		String panel = "(56151,63031,65535)"
		String sub = "(49151,60031,65535)"
		String buttons = "(4768,34615,65535)"
		String line = "(65535,49151,55704)"
		String chi = "(65535,65533,32768)"
	elseif (Colors == 1)
		back = "(4*65535/256,48*65535/256,70*65535/256)"
		panel = "(182*65535/256,244*65535/256,110*65535/256)"
		sub = "(124*65535/256,231*65535/256,0*65535/256)"
		buttons = "(256*65535/256,54*65535/256,0*65535/256)"
		line = "(256*65535/256,105*65535/256,65*65535/256)"
		chi = "(100*65535/256,174*65533/256,208*65533/256)"
	elseif (Colors == 2)
		back = "(0*65535/256,0*65535/256,102*65535/256)"
		panel = "(255*65535/256,255*65535/256,204*65535/256)"
		sub = "(204*65535/256,204*65535/256,255*65535/256)"
		buttons = "(153*65535/256,0*65535/256,102*65535/256)"
		line = "(153*65535/256,0*65535/256,102*65535/256)"
		chi = "(255*65535/256,255*65535/256,204*65535/256)"
	elseif (Colors == 3)
		back = "(0*65535/256,99*65535/256,150*65535/256)"
		panel = "(198*65535/256,240*65535/256,254*65535/256)"
		sub = "(156*65535/256,226*65535/256,252*65535/256)"
		buttons = "(190*65535/256,33*65535/256,50*65535/256)"
		line = "(255*65535/256,51*65535/256,77*65535/256)"
		chi = "(256*65535/256,256*65535/256,256*65535/256)"
	endif
	// first make a new data folder
	NewDataFolder/O/S root:Analysis:Global
	Variable/G StartsS=0, EndsS=1
	Variable/G StartsD=Show, EndsD=Show+1
	Variable/G StartsT=Show, EndsT=Show+1
	Variable/G IncludeSFlag=1, IncludeDFlag=1, IncludeTFlag=1
	Variable/G Tau1Start=3, Tau2Start=1.2, Tau3Start=2, FixTau1=0, FixTau2=0, FixBeta=0
	PauseUpdate; Silent 1		// building window...
	DoWindow/F/K GlobalFit
	NewPanel/K=1/W=(300,100,670,290)
	DoWindow/C GlobalFit
	Execute("ModifyPanel cbRGB=" + panel)
	CheckBox checkIncS,		pos={10,15},title="Include single-exp data:",fsize=13,variable=root:Analysis:Global:IncludeSFlag
	SetVariable setvarStartS,	pos={180,15}, title=" ",	size={80,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:StartsS
	SetVariable setvarEndS,		pos={260,15}, title="to",	size={100,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:EndsS
	CheckBox checkIncD,		pos={10,40},title="Include double-exp data:",fsize=13,variable=root:Analysis:Global:IncludeDFlag
	SetVariable setvarStartD,	pos={180,40}, title=" ",	size={80,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:StartsD
	SetVariable setvarEndD,		pos={260,40}, title="to",	size={100,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:EndsD
	CheckBox checkIncT,		pos={10,65},title="Include triple-exp data:",fsize=13,variable=root:Analysis:Global:IncludeTFlag
	SetVariable setvarStartT,	pos={180,65}, title=" ",	size={80,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:StartsT
	SetVariable setvarEndT,		pos={260,65}, title="to",	size={100,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:EndsT
	CheckBox checkTau1,		pos={10,90},title="Fix \F'Symbol't\F'Arial'\B1",fsize=13,variable=root:Analysis:Global:FixTau1
	CheckBox checkTau2,		pos={80,90},title="Fix \F'Symbol't\F'Arial'\B2",fsize=13,variable=root:Analysis:Global:FixTau2
	CheckBox checkBeta,		pos={160,90},title="Fix \F'Symbol'b\F'Arial' ",fsize=13,variable=root:Analysis:Global:FixBeta
	SetVariable setvarTau1,		pos={10,115}, title="initial \F'Symbol't\F'Arial'\B1",	size={115,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:Tau1Start	
	SetVariable setvarTau2,		pos={130,115}, title="initial \F'Symbol't\F'Arial'\B2",	size={115,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:Tau2Start
	SetVariable setvarTau3,		pos={250,115}, title="initial \F'Symbol't\F'Arial'\B3",	size={115,20},limits={0,1000,1},fsize=13,value=root:Analysis:Global:Tau3Start
	Button buttonFit,pos={100,150},size={140,30},fsize=13,proc=GlobalFitHistograms,title="Fit linking lifetimes"
	Execute("Button buttonFit,fColor="+buttons)
End

// GlobalFitHistograms() ***********************************
Function GlobalFitHistograms(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Analysis:Global
	NVAR StartsS, EndsS, StartsD, EndsD, StartsT, EndsT, Tau1Start, Tau2Start, Tau3Start, FixTau1, FixTau2, FixBeta
	NVAR  IncludeSFlag, IncludeDFlag, IncludeTFlag
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	Wave TrialS = root:Analysis:Manual:TrialValuesSingle
	Wave TrialD = root:Analysis:Manual:TrialValuesDouble
	Wave TrialT = root:Analysis:Manual:TrialValuesTriple
	Make/O/T FitFuncNames = {"Three_Lifetimes"}
	Variable N = 0		// number of datasets to be fit
	if (IncludeSFlag == 1)
		N += EndsS - StartsS + 1
	endif
	if (IncludeDFlag == 1)
		N += EndsD - StartsD + 1
	endif
	if (IncludeTFlag == 1)
		N += EndsT - StartsT + 1
	endif
	
	// Wave containing a list of data sets.
	Make/O/N=(N,3)/T DataSets	
	SetDimLabel 1, 2, Weights, DataSets		// A column with label "Weights", if it exists, contains names of weighting waves for each dataset.
	DataSets[][1] = "_calculated_"			// Column 1 contains X data sets. Enter _calculated_ in a row if appropriate.
	Variable counter = 0, i
	if (IncludeSFlag == 1)													
		for (i = StartsS; i<=EndsS; i += 1)
			DataSets[counter][0] = "root:Data:hist_" + num2str(i)	// Column 0 contains Y data sets.
			DataSets[counter][2] = "root:Data:weigh_" + num2str(i)
			counter += 1
		endfor 
	endif
	if (IncludeDFlag == 1)	
		for (i = StartsD; i<=EndsD; i += 1)
			DataSets[counter][0] = "root:Data:hist_" + num2str(i)
			DataSets[counter][2] = "root:Data:weigh_" + num2str( i)
			counter += 1
		endfor 
	endif
	if (IncludeTFlag == 1)	
		for (i = StartsT; i<=EndsT; i += 1)
			DataSets[counter][0] = "root:Data:hist_" + num2str(i)
			DataSets[counter][2] = "root:Data:weigh_" + num2str( i)
			counter += 1
		endfor 
	endif
	
	// a matrix wave with a row for each data set and N+4 columns, where N is the maximum number of coefficients
	Make/O/N=(N, 18) CoefDataSetLinkage	
	CoefDataSetLinkage[][0] = 0	// all datasets use the same fit function
	CoefDataSetLinkage[][1] = 0	// set by global fit function
	CoefDataSetLinkage[][2] = 0	// set by global fit function
	CoefDataSetLinkage[][3] = 14	// Two_Lifetimes needs 14  coefficients
	
	//		|	f()	first	last	N		a	tg	sig	dt	r	th1	th2	b	t1	t2  beta t3 gamma	SyncP	
	//		|	0	1		2	3		4	5	6	7	8	9	10	11	12	13	14	15	16		17	
	//	---|----------------------------------------------------------------
	//	ds1	|	0	0		0	12		0	1	2	3	4	5	6	7	8	9	10	11	12		13
	//	ds2	|	0	0		0	12		14	15	16	17	18	19	20	21	8	9	22	11	23		24
	//	ds3	|	0	0		0	12		25	26	27	28	29	30	31	32	8	9	33	11	34		35
	//	ds4	|	0	0		0	12		36	37	38	39	40	41	42	43	8	9	44	11	45		46
	
	CoefDataSetLinkage[][4] = p*11+3		// "S" column
	CoefDataSetLinkage[0][4] = 0
	
	CoefDataSetLinkage[][5] = CoefDataSetLinkage[p][4] +1	// tg
	CoefDataSetLinkage[][6] = CoefDataSetLinkage[p][5] +1	//sigma
	CoefDataSetLinkage[][7] = CoefDataSetLinkage[p][6] +1	// deltat
	CoefDataSetLinkage[][8] = CoefDataSetLinkage[p][7] +1	// rho
	CoefDataSetLinkage[][9] = CoefDataSetLinkage[p][8] +1	// theta1
	CoefDataSetLinkage[][10] = CoefDataSetLinkage[p][9] +1	// theta2
	CoefDataSetLinkage[][11] = CoefDataSetLinkage[p][10] +1	//b
	
	CoefDataSetLinkage[][12] = 8	// tau1
	CoefDataSetLinkage[][13] = 9	// tau2
	
	CoefDataSetLinkage[][14] = CoefDataSetLinkage[p][11] +1	// beta
	CoefDataSetLinkage[0][14] = 10
	
	CoefDataSetLinkage[][15] = 11	// tau3
	
	CoefDataSetLinkage[][16] = CoefDataSetLinkage[p][14] +1	// gamma
	CoefDataSetLinkage[0][16] = 12
	
	CoefDataSetLinkage[][17] = CoefDataSetLinkage[p][16] +1	// SyncPeriod
	
	WaveStats/Q/M=1 CoefDataSetLinkage
	
	// Wave containing initial guesses. The entries in the fifth and greater columns of the CoefDataSetLinkage wave are indices into this wave.
	Make/O/N=(V_max+1,3) CoefWave=0		
	SetDimLabel 1, 1, Hold, CoefWave
	SetDimLabel 1, 2, Epsilon, CoefWave
	CoefWave[][2] = 1e-06		// epsilon values
	counter = 0
	if (IncludeSFlag == 1)													
		for (i = StartsS; i<=EndsS; i += 1)				
			CoefWave[counter][0] = TrialS[i][%S]; counter += 1		// Column 0 contains initial guesses		
			CoefWave[counter][0] = TrialS[i][%tg]	; counter += 1
			CoefWave[counter][0] = TrialS[i][%sigma]	; counter += 1
			CoefWave[counter][0] = TrialS[i][%deltat]	; counter += 1
			CoefWave[counter][0] = TrialS[i][%rho]	; counter += 1
			CoefWave[counter][0] = TrialS[i][%theta1]	; counter += 1
			CoefWave[counter][0] = TrialS[i][%theta2]	; counter += 1
			CoefWave[counter][0] = TrialS[i][%b]	; counter += 1
			if (counter == 8)
				CoefWave[counter][0] = Tau1Start; counter += 1	// tau1
				CoefWave[counter][0] = Tau2Start; counter += 1	// tau2
				if (IncludeDFlag == 0 && IncludeTFlag == 0)	//double and triple-exp datasets are not included in fit
					CoefWave[counter-1][1] = 1
				endif
			endif
			CoefWave[counter][0] = 0	; counter += 1	// beta = 0 for single-exp datasets
			CoefWave[counter-1][1] = 1				// hold beta=0
			if (counter == 11)
				CoefWave[counter][0] = Tau3Start; counter += 1	// tau3
				if (IncludeTFlag == 0)	// triple-exp datasets are not included in fit
					CoefWave[counter-1][1] = 1
				endif
			endif
			CoefWave[counter][0] = 0	; counter += 1	// gamma = 0 for single-exp datasets
			CoefWave[counter-1][1] = 1				// hold gamma=0
			CoefWave[counter][0] = FileInfo_Variables[i][%SyncPeriod]	; counter += 1	
			CoefWave[counter-1][1] = 1				// always hold SyncPeriod
		endfor
	endif
	if (IncludeDFlag == 1)													
		for (i = StartsD; i<=EndsD; i += 1)				
			CoefWave[counter][0] = TrialD[i][%S]; counter += 1		// Column 0 contains initial guesses		
			CoefWave[counter][0] = TrialD[i][%tg]	; counter += 1
			CoefWave[counter][0] = TrialD[i][%sigma]	; counter += 1
			CoefWave[counter][0] = TrialD[i][%deltat]	; counter += 1
			CoefWave[counter][0] = TrialD[i][%rho]	; counter += 1
			CoefWave[counter][0] = TrialD[i][%theta1]; counter += 1
			CoefWave[counter][0] = TrialD[i][%theta2]; counter += 1
			CoefWave[counter][0] = TrialD[i][%b]	; counter += 1
			if (counter == 8)
				CoefWave[counter][0] = Tau1Start					// tau1
				CoefWave[counter][1] = FixTau1; counter += 1		// if desired fix tau1
				CoefWave[counter][0] = Tau2Start; counter += 1	// tau2
			endif
			CoefWave[counter][0] = TrialD[i][%bet]; counter += 1	
			if (counter == 11)
				CoefWave[counter][0] = Tau3Start;counter += 1	// tau3
				if (IncludeTFlag == 0)	// triple-exp datasets are not included in fit
					CoefWave[counter-1][1] = 1
				endif
			endif
			CoefWave[counter][0] = 0	; counter += 1	// gamma = 0 for single-exp datasets
			CoefWave[counter-1][1] = 1				// hold gamma=0
			CoefWave[counter][0] = FileInfo_Variables[i][%SyncPeriod]; counter += 1	
			CoefWave[counter-1][1] = 1				// always hold SyncPeriod
		endfor
	endif
	if (IncludeTFlag == 1)													
		for (i = StartsT; i<=EndsT; i += 1)				
			CoefWave[counter][0] = TrialT[i][%S]; counter += 1		// Column 0 contains initial guesses		
			CoefWave[counter][0] = TrialT[i][%tg]	; counter += 1
			CoefWave[counter][0] = TrialT[i][%sigma]	; counter += 1
			CoefWave[counter][0] = TrialT[i][%deltat]	; counter += 1
			CoefWave[counter][0] = TrialT[i][%rho]	; counter += 1
			CoefWave[counter][0] = TrialT[i][%theta1]; counter += 1
			CoefWave[counter][0] = TrialT[i][%theta2]; counter += 1
			CoefWave[counter][0] = TrialT[i][%b]	; counter += 1
			if (counter == 8)
				CoefWave[counter][0] = Tau1Start					// tau1
				CoefWave[counter][1] = FixTau1; counter += 1		// if desired fix tau1
				CoefWave[counter][0] = Tau2Start					// tau2
				CoefWave[counter][1] = FixTau2; counter += 1		// if desired fix tau2
			endif
			CoefWave[counter][0] = TrialT[i][%bet]
			CoefWave[counter][1] = FixBeta; counter += 1		// if desired fix beta
			if (counter == 11)
				CoefWave[counter][0] = Tau3Start;counter += 1	// tau3
			endif
			CoefWave[counter][0] = TrialT[i][%gamm]; counter += 1
			CoefWave[counter][0] = FileInfo_Variables[i][%SyncPeriod]; counter += 1	
			CoefWave[counter-1][1] = 1				// always hold SyncPeriod
		endfor
	endif
	
	// This constraint wave will be used straight as it comes, so K0, K1, etc. refer to the order of coefficients as laid out in CoefWave.
	Make/O/T/N=(2*counter) ConstraintWave=""	
	counter = 0
	Variable Kvalue = 0
	if (IncludeSFlag == 1)													
		for (i = StartsS; i<=EndsS; i += 1)				
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLS])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULS]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLtg])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULtg]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLsigma])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULsigma]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLdeltat])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULdeltat]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLrho])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULrho]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLtheta1])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULtheta1]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLtheta2])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULtheta2]); counter += 1; Kvalue += 1	
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLb])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULb]); counter += 1; Kvalue += 1
			if (Kvalue == 8)
				ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialS[i][%LLtau1])	
				ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialS[i][%ULtau1]); counter += 1; Kvalue += 1
				Kvalue += 1	// don't constrain tau2
			endif
			Kvalue += 1	// don't constrain beta
			if (Kvalue == 11)
				Kvalue += 1	// don't constrain tau3
			endif
			Kvalue += 2	// don't constrain gamma, SyncPeriod
		endfor
	endif
	if (IncludeDFlag == 1)													
		for (i = StartsD; i<=EndsD; i += 1)				
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLS])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULS]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLtg])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULtg]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLsigma])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULsigma]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLdeltat])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULdeltat]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLrho])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULrho]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLtheta1])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULtheta1]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLtheta2])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULtheta2]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLb])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULb]); counter += 1; Kvalue += 1
			if (Kvalue == 8)
				if (FixTau1 == 0)		// only constrain tau1 if it's not fixed
					ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLtau1])	
					ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULtau1]); counter += 1; Kvalue += 1
				else
					Kvalue += 1
				endif
				ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLtau2])	
				ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULtau2]); counter += 1; Kvalue += 1
			endif
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialD[i][%LLbet])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialD[i][%ULbet]); counter += 1; Kvalue += 1
			if (Kvalue == 11)
				Kvalue += 1	// don't constrain tau3
			endif
			Kvalue += 2	// don't constrain gamma, SyncPeriod
		endfor
	endif	
	if (IncludeTFlag == 1)													
		for (i = StartsD; i<=EndsD; i += 1)				
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLS])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULS]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLtg])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULtg]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLsigma])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULsigma]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLdeltat])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULdeltat]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLrho])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULrho]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLtheta1])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULtheta1]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLtheta2])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULtheta2]); counter += 1; Kvalue += 1
			ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLb])	
			ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULb]); counter += 1; Kvalue += 1
			if (Kvalue == 8)
				if (FixTau1 == 0)		// only constrain tau1 if it's not fixed
					ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLtau1])	
					ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULtau1]); counter += 1; Kvalue += 1
				else
					Kvalue += 1
				endif
				if (FixTau2 == 0)		// only constrain tau2 if it's not fixed
					ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLtau2])	
					ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULtau2]); counter += 1; Kvalue += 1
				else
					Kvalue += 1
				endif
			endif
			if (FixBeta == 0)		// only constrain beta if it's not fixed
				ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLbet])	
				ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULbet]); counter += 1; Kvalue += 1
			else
				Kvalue += 1
			endif
			if (Kvalue == 11)
				ConstraintWave	[2*counter] 	= "K" + num2str(Kvalue) + ">" + num2str(TrialT[i][%LLtau3])	
				ConstraintWave	[2*counter+1] = "K" + num2str(Kvalue) + "<" + num2str(TrialT[i][%ULtau3]); counter += 1; Kvalue += 1
			endif
			Kvalue += 2	// don't constrain gamma, SyncPeriod
		endfor
	endif	
	Redimension/N=(2*counter) ConstraintWave
	
	Variable Options = 239	// 1+2+4+8+32+64+128
	Variable FitCurvePoints = numpnts(root:Data:hist_0)	// number of points for auto-destination waves
	Variable DoAlertsOnError = 1
	
	DoNewGlobalFit(FitFuncNames, DataSets, CoefDataSetLinkage, CoefWave, $"", ConstraintWave, Options, FitCurvePoints, DoAlertsOnError)
	
	// Now copy results of the fit into the FitValues tables
	Wave FitS = root:Analysis:Fits:FitValuesSingle
	Wave FitD = root:Analysis:Fits:FitValuesDouble
	Wave FitT = root:Analysis:Fits:FitValuesTriple
	Wave MasterCoefs = root:Packages:NewGlobalFit:MasterCoefs
	Wave Sigma = root:Packages:NewGlobalFit:W_sigma
	
	counter = 0
	if (IncludeSFlag == 1)													
		for (i = StartsS; i<=EndsS; i += 1)			
			FitS[i][%S] = MasterCoefs[counter]
			FitS[i][	%SError] = Sigma[counter]; counter += 1
			FitS[i][%tg] = MasterCoefs[counter]
			FitS[i][	%tgError] = Sigma[counter]; counter += 1
			FitS[i][%sigma] = MasterCoefs[counter]
			FitS[i][	%sigmaError] = Sigma[counter]; counter += 1
			FitS[i][%deltat] = MasterCoefs[counter]
			FitS[i][	%deltatError] = Sigma[counter]; counter += 1
			FitS[i][%rho] = MasterCoefs[counter]
			FitS[i][	%rhoError] = Sigma[counter]; counter += 1
			FitS[i][%theta1] = MasterCoefs[counter]
			FitS[i][	%theta1Error] = Sigma[counter]; counter += 1
			FitS[i][%theta2] = MasterCoefs[counter]
			FitS[i][	%theta2Error] = Sigma[counter]; counter += 1			
			FitS[i][%b] = MasterCoefs[counter]
			FitS[i][	%bError] = Sigma[counter]; counter += 1
			if (counter == 8)
				counter += 6	// skip tau1, tau2, beta, tau3, gamma, SyncPeriod
			else
				counter += 3	// skip beta, gamma, SyncPeriod
			endif
			// always the same
			FitS[i][%tau1] = MasterCoefs[8]
			FitS[i][	%tau1Error] = Sigma[8]
		endfor
	endif
	if (IncludeDFlag == 1)													
		for (i = StartsD; i<=EndsD; i += 1)				
			FitD[i][%S] = MasterCoefs[counter]
			FitD[i][%SError] = Sigma[counter]; counter += 1
			FitD[i][%tg] = MasterCoefs[counter]
			FitD[i][%tgError] = Sigma[counter]; counter += 1
			FitD[i][%sigma] = MasterCoefs[counter]
			FitD[i][%sigmaError] = Sigma[counter]; counter += 1
			FitD[i][%deltat] = MasterCoefs[counter]
			FitD[i][%deltatError] = Sigma[counter]; counter += 1
			FitD[i][%rho] = MasterCoefs[counter]
			FitD[i][%rhoError] = Sigma[counter]; counter += 1
			FitD[i][%theta1] = MasterCoefs[counter]
			FitD[i][%theta1Error] = Sigma[counter]; counter += 1
			FitD[i][%theta2] = MasterCoefs[counter]
			FitD[i][%theta2Error] = Sigma[counter]; counter += 1			
			FitD[i][%b] = MasterCoefs[counter]
			FitD[i][%bError] = Sigma[counter]; counter += 1
			if (counter == 8)
				counter += 2	// skip tau1, tau2 
			endif
			// always the same
			FitD[i][%tau1] = MasterCoefs[8]
			FitD[i][%tau1Error] = Sigma[8]
			FitD[i][%tau2] = MasterCoefs[9]
			FitD[i][%tau2Error] = Sigma[9]
			
			FitD[i][%bet] = MasterCoefs[counter]
			FitD[i][%betError] = Sigma[counter]; counter += 1
			if (counter == 11)
				counter += 1	// skip tau3
			endif
			counter += 2	// skip gamma, SyncPeriod
		endfor
	endif
	if (IncludeTFlag == 1)													
		for (i = StartsT; i<=EndsT; i += 1)				
			FitT[i][%S] = MasterCoefs[counter]
			FitT[i][%SError] = Sigma[counter]; counter += 1
			FitT[i][%tg] = MasterCoefs[counter]
			FitT[i][%tgError] = Sigma[counter]; counter += 1
			FitT[i][%sigma] = MasterCoefs[counter]
			FitT[i][%sigmaError] = Sigma[counter]; counter += 1
			FitT[i][%deltat] = MasterCoefs[counter]
			FitT[i][%deltatError] = Sigma[counter]; counter += 1
			FitT[i][%rho] = MasterCoefs[counter]
			FitT[i][%rhoError] = Sigma[counter]; counter += 1
			FitT[i][%theta1] = MasterCoefs[counter]
			FitT[i][%theta1Error] = Sigma[counter]; counter += 1
			FitT[i][%theta2] = MasterCoefs[counter]
			FitT[i][%theta2Error] = Sigma[counter]; counter += 1			
			FitT[i][%b] = MasterCoefs[counter]
			FitT[i][%bError] = Sigma[counter]; counter += 1
			if (counter == 8)
				counter += 2	// skip tau1, tau2 
			endif
			// always the same
			FitT[i][%tau1] = MasterCoefs[8]
			FitT[i][%tau1Error] = Sigma[8]
			FitT[i][%tau2] = MasterCoefs[9]
			FitT[i][%tau2Error] = Sigma[9]
			
			FitT[i][%bet] = MasterCoefs[counter]
			FitT[i][%betError] = Sigma[counter]; counter += 1
			if (counter == 11)
				counter += 1	// skip tau3
			endif
			// always the same
			FitT[i][%tau3] = MasterCoefs[9]
			FitT[i][%tau3Error] = Sigma[9]
			
			FitT[i][%gamm] = MasterCoefs[counter]
			FitT[i][%gammError] = Sigma[counter]; counter += 1
			counter += 1	// skip SyncPeriod
		endfor
	endif
	// overwrite fits
	if (IncludeSFlag == 1)													
		for (i = StartsS; i<=EndsS; i += 1)			
			Wave GFit = $("root:Data:Gfit_hist_" + num2str(i))
			String Fit = "root:Analysis:Fits:Single:SingleFit_" + num2str(i)
			Duplicate/O GFit, $Fit
			Wave GRes = $("root:Data:GRes_hist_" + num2str(i))
			String Res = "root:Analysis:Fits:Single:Res_SingleFit_" + num2str(i)
			SetScale/P x 0, FileInfo_Variables[i][%Binsize],GRes
			Duplicate/O GRes, $Res
			// divide residual by standard deviation
			String NormRes = "root:Analysis:Fits:Single:NormRes_SingleFit_" + num2str(i)
			Wave Weigh = $("root:Data:weigh_" + num2str(i))
			GRes /= Weigh
			Duplicate/O GRes, $NormRes	
			//			KillWaves GFit, GRes, $("root:Data:Coef_hist_" + num2str(i))
		endfor
	endif
	if (IncludeDFlag == 1)													
		for (i = StartsD; i<=EndsD; i += 1)			
			Wave GFit = $("root:Data:Gfit_hist_" + num2str(i))
			Fit = "root:Analysis:Fits:Double:DoubleFit_" + num2str(i)
			Duplicate/O GFit, $Fit
			Wave GRes = $("root:Data:GRes_hist_" + num2str(i))
			Res = "root:Analysis:Fits:Double:Res_DoubleFit_" + num2str(i)
			SetScale/P x 0, FileInfo_Variables[i][%Binsize],GRes
			Duplicate/O GRes, $Res
			// divide residual by standard deviation
			NormRes = "root:Analysis:Fits:Double:NormRes_DoubleFit_" + num2str(i)
			Wave Weigh = $("root:Data:weigh_" + num2str(i))
			GRes /= Weigh
			Duplicate/O GRes, $NormRes	
			//		KillWaves GFit, GRes, $("root:Data:Coef_hist_" + num2str(i))
		endfor
	endif
	if (IncludeTFlag == 1)													
		for (i = StartsT; i<=EndsT; i += 1)			
			Wave GFit = $("root:Data:Gfit_hist_" + num2str(i))
			Fit = "root:Analysis:Fits:Triple:TripleFit_" + num2str(i)
			Duplicate/O GFit, $Fit
			Wave GRes = $("root:Data:GRes_hist_" + num2str(i))
			Res = "root:Analysis:Fits:Triple:Res_TripleFit_" + num2str(i)
			SetScale/P x 0, FileInfo_Variables[i][%Binsize],GRes
			Duplicate/O GRes, $Res
			// divide residual by standard deviation
			NormRes = "root:Analysis:Fits:Triple:NormRes_TripleFit_" + num2str(i)
			Wave Weigh = $("root:Data:weigh_" + num2str(i))
			GRes /= Weigh
			Duplicate/O GRes, $NormRes	
			//		KillWaves GFit, GRes, $("root:Data:Coef_hist_" + num2str(i))
		endfor
	endif
	DoUpdate
End

//*****************************************************************************************//
//********************************************** Auxiliary Functions *****************************//
//*****************************************************************************************//

Function KeyboardPanelHook(s)	// Capture keyboard events in Analysis panel. Created by Rodofo Hermans
	STRUCT WMWinHookStruct &s
	NVAR Show = root:Data:Show	
	NVAR MaxData = root:Data:MaxData
	if(s.eventCode==11)
		switch(s.keycode)
			case 31:		//key arrow down 
				if(Show > 0)
					Show -=1
					DisplayData("",0,"","")
				endif
				break
			case 30:		//key arrow up
				if (Show <= MaxData-2)
					Show +=1
					DisplayData("",0,"","")
				endif
				break
		endswitch
	endif
	return 0
End

Function MyTabControl(TC_Struct) : TabControl
	STRUCT WMTabControlAction &TC_Struct
	NVAR Show = root:Data:Show
	Wave Option = root:Analysis:SelectedTab
	if (TC_Struct.tab != 0 && TC_Struct.tab != 5)
		Option[Show] = TC_Struct.tab
	endif
	ACL_DisplayTab(TC_Struct)
	ShowCurve("",0,"","")
End

// CheckLimits *********************************
Function CheckLimits(option)
	Variable option
	NVAR Show = root:Data:Show
	if (option == 1)
		Wave Trial = root:Analysis:Manual:TrialValuesIRF
	elseif (option == 2)
		Wave Trial = root:Analysis:Manual:TrialValuesSingle
	elseif  (option == 3)
		Wave Trial = root:Analysis:Manual:TrialValuesDouble
	else
		Wave Trial = root:Analysis:Manual:TrialValuesTriple
	endif
	if (Trial[Show][%S] < Trial[Show][%LLS]) 
		Trial[Show][%S] = Trial[Show][%LLS] + 0.00001
	endif
	if (Trial[Show][%S] > Trial[Show][%ULS])
		Trial[Show][%S]= Trial[Show][%ULS] - 0.00001
	endif
	if (Trial[Show][%tg] < Trial[Show][%LLtg]) 
		Trial[Show][%tg] = Trial[Show][%LLtg] + 0.00001
	endif
	if (Trial[Show][%tg] > Trial[Show][%ULtg])
		Trial[Show][%tg] = Trial[Show][%ULtg] - 0.00001
	endif
	if (Trial[Show][%sigma] < Trial[Show][%LLsigma]) 
		Trial[Show][%sigma] = Trial[Show][%LLsigma] + 0.00001
	endif
	if (Trial[Show][%sigma] > Trial[Show][%ULsigma])
		Trial[Show][%sigma] = Trial[Show][%ULsigma] - 0.00001
	endif
	if (Trial[Show][%deltat] < Trial[Show][%LLdeltat]) 
		Trial[Show][%deltat] = Trial[Show][%LLdeltat] + 0.00001
	endif
	if (Trial[Show][%deltat] > Trial[Show][%ULdeltat])
		Trial[Show][%deltat] = Trial[Show][%ULdeltat] - 0.00001
	endif
	if (abs(Trial[Show][%deltat]) < 1e-6)
		Trial[Show][%deltat] = 1e-6 * sign(Trial[Show][%deltat])
	endif
	if (Trial[Show][%rho] < Trial[Show][%LLrho]) 
		Trial[Show][%rho] = Trial[Show][%LLrho] + 0.00001
	endif
	if (Trial[Show][%rho] > Trial[Show][%ULrho])
		Trial[Show][%rho] = Trial[Show][%ULrho] - 0.00001
	endif
	if (Trial[Show][%theta1] < Trial[Show][%LLtheta1]) 
		Trial[Show][%theta1] =Trial[Show][% LLtheta1] + 0.00001
	endif
	if (Trial[Show][%theta1] > Trial[Show][%ULtheta1])
		Trial[Show][%theta1] = Trial[Show][%ULtheta1] - 0.00001
	endif
	if (Trial[Show][%theta2] < Trial[Show][%LLtheta2]) 
		Trial[Show][%theta2] = Trial[Show][%LLtheta2] + 0.00001
	endif
	if (Trial[Show][%theta2] > Trial[Show][%ULtheta2])
		Trial[Show][%theta2] = Trial[Show][%ULtheta2] - 0.00001
	endif
	if (Trial[Show][%b] < Trial[Show][%LLb]) 
		Trial[Show][%b] = Trial[Show][%LLb] + 0.00001
	endif
	if (Trial[Show][%b] > Trial[Show][%ULb])
		Trial[Show][%b] = Trial[Show][%ULb] - 0.00001
	endif
	if (option > 1 )		// single or double or triple
		if (Trial[Show][%tau1] < Trial[Show][%LLtau1]) 
			Trial[Show][%tau1] = Trial[Show][%LLtau1] + 0.00001
		endif
		if (Trial[Show][%tau1] > Trial[Show][%ULtau1])
			Trial[Show][%tau1] = Trial[Show][%ULtau1] - 0.00001
		endif
	endif
	if (option == 3 || option == 4)	// double or triple
		if (Trial[Show][%tau2] < Trial[Show][%LLtau2]) 
			Trial[Show][%tau2] = Trial[Show][%LLtau2] + 0.00001
		endif
		if (Trial[Show][%tau2] > Trial[Show][%ULtau2])
			Trial[Show][%tau2] = Trial[Show][%ULtau2] - 0.00001
		endif
		if (Trial[Show][%bet] < Trial[Show][%LLbet]) 
			Trial[Show][%bet] = Trial[Show][%LLbet] + 0.00001
		endif
		if (Trial[Show][%bet] > Trial[Show][%ULbet])
			Trial[Show][%bet] = Trial[Show][%ULbet] - 0.00001
		endif
	endif
	if (option == 4)		// triple
		if (Trial[Show][%tau3] < Trial[Show][%LLtau3]) 
			Trial[Show][%tau3] = Trial[Show][%LLtau3] + 0.00001
		endif
		if (Trial[Show][%tau3] > Trial[Show][%ULtau3])
			Trial[Show][%tau3] = Trial[Show][%ULtau3] - 0.00001
		endif
		if (Trial[Show][%gamm] < Trial[Show][%LLgamm]) 
			Trial[Show][%gamm] = Trial[Show][%LLgamm] + 0.00001
		endif
		if (Trial[Show][%gamm] > Trial[Show][%ULgamm])
			Trial[Show][%gamm] = Trial[Show][%ULgamm] - 0.00001
		endif
	endif
End

// ChangeRange *********************************************
Function ChangeRange (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	// value of variable as number
	String varStr		// value of variable as string
	String varName	// name of variable
	CreateWeighted()
	DisplayPixelAnalysisResults("",1)
	DisplayWeighted("",1)
End

// CheckFix *********************************************
Function CheckFix (ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked			// 1 if selelcted, 0 if not
	NVAR Show = root:Data:Show
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	if (Option == 1)
		String Trial = "root:Analysis:Manual:TrialValuesIRF"
		// now get the end characters of the string
		String Name = StringByKey("checkIR", ctrlName, "F")
	elseif (Option == 2)
		Trial = "root:Analysis:Manual:TrialValuesSingle"
		Name = StringByKey("check", ctrlName, "S")
	elseif (Option == 3)
		Trial = "root:Analysis:Manual:TrialValuesDouble"
		Name = StringByKey("check", ctrlName, "D")
	elseif (Option == 4)
		Trial = "root:Analysis:Manual:TrialValuesTriple"
		Name = StringByKey("check", ctrlName, "T")
	endif
	Execute(Trial + "[" + num2str(Show) + "][%" + Name + "] =" + num2str(checked))
End

// CheckFixUpdatePanel ******************************************
Function CheckFixUpdatePanel()
	NVAR Show = root:Data:Show
	Variable i
	DoWindow/F FLIMPanel 
	Make/O/T values={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b"}	
	for (i = 0; i < numpnts(values); i += 1)	
		Execute("CheckBox checkIRFFix" + values[i] + ", value = root:Analysis:Manual:TrialValuesIRF[" + num2str(Show) + "][% Fix" + values[i] + "]"  )
	endfor
	Make/O/T values={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1"}	
	for (i = 0; i < numpnts(values); i += 1)	
		Execute("CheckBox checkSFix" + values[i] + ", value = root:Analysis:Manual:TrialValuesSingle[" + num2str(Show) + "][% Fix" + values[i] + "]"  )
	endfor
	Make/O/T values={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1", "tau2", "bet"}	
	for (i = 0; i < numpnts(values); i += 1)	
		Execute("CheckBox checkDFix" + values[i] + ", value = root:Analysis:Manual:TrialValuesDouble[" + num2str(Show) + "][% Fix" + values[i] + "]"  )
	endfor
	Make/O/T values={"S", "tg", "sigma", "deltat","rho","theta1","theta2","b", "tau1", "tau2", "bet","tau3","gamm"}	
	for (i = 0; i < numpnts(values); i += 1)	
		Execute("CheckBox checkTFix" + values[i] + ", value = root:Analysis:Manual:TrialValuesTriple[" + num2str(Show) + "][% Fix" + values[i] + "]"  )
	endfor
	KillWaves/Z values
End

// BinningUpdatePanel ******************************************
Function BinningUpdatePanel()
	SetDataFolder root:Data
	NVAR Show	
	Wave FileInfo_Variables
	Variable Binning = FileInfo_Variables[Show][%Binning]
	DoWindow/F FLIMPanel 
	if (Binning == 1)
		Variable m = 1
	elseif (Binning == 2)
		m = 2
	elseif (Binning == 4)
		m = 3
	elseif (Binning == 8)
		m = 4
	elseif (Binning == 16)
		m = 5
	elseif (Binning == 32)
		m = 6
	elseif (Binning == 64)
		m = 7
	elseif (Binning == 128)
		m = 8
	elseif (Binning == 256)
		m = 9
	elseif (Binning == 512)
		m = 10
	endif
	PopupMenu popupSBinning, mode = m
	PopupMenu popupDBinning, mode = m
End

// SelectChannel *******************************************
Function SelectChannel(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	FileInfo_Variables[%Displayed][%Channel] = popNum - 1			
End

// SetBinning *********************************
Function SetBinning(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	NVAR Show = root:Data:Show
	Wave SelectedTab = root:Analysis:SelectedTab
	Variable Option = SelectedTab[Show]
	if (Option == 1)
		ctrlName = "setvarIRF"
	elseif(Option == 2)
		ctrlName = "setvarS"
	elseif(Option == 3)
		ctrlName = "setvarD"
	endif
	Wave FileInfo_Variables = root:Data:FileInfo_Variables
	FileInfo_Variables[%Displayed][%Binning] = 2^(popNum-1)			
	DisplayData(ctrlName, 0, "", "")			
End

//*****************************************************************************************//
//********************************************** Calculate Functions *****************************//
//*****************************************************************************************//

// CalculateIRF *********************************
Function CalculateIRF(a, tg, sigma, deltat, rho,  theta1, theta2, b, Hist)
	Variable a, tg, sigma, deltat, rho,  theta1, theta2, b
	Wave Hist
	Duplicate/O Hist, tmtb
	Variable tb = tg + deltat
	tmtb= x-tb
	Variable theta12 = theta1/theta2
	
	Hist = a * Exp( -(x-tg)^2/2/sigma^2)
	Hist += a*rho * (1+1/theta12) * (1+theta12)^(1/theta12)* (Sign(tmtb)+1)/2 * Exp(-tmtb/theta1) * (1- Exp(-tmtb/theta2) ) 
	Hist /= Sqrt(2*Pi)*sigma + rho*theta1*(theta12+1)^(1/theta12)
	Hist += b		
	KillWaves/Z tmtb
End

// CalculateComponents *********************************
Function CalculateComponents(a, tg, sigma, tb, rho,  theta1, theta2, b, Gaussian, C1, C2, Back)
	Variable a, tg, sigma, tb, rho,  theta1, theta2, b
	Wave Gaussian, C1, C2, Back
	Duplicate/O Gaussian, tmtb 
	tmtb= x-tb
	Variable theta12 = theta1/theta2
	Gaussian = a * Exp( -(x-tg)^2/2/sigma^2)
	C1 = a*rho * (1+1/theta12) * (1+theta12)^(1/theta12)* (Sign(tmtb)+1)/2 * Exp(-tmtb/theta1) * (1- Exp(-tmtb/theta2) ) 
	Back = b
	KillWaves/Z tmtb
End

// Calc2Lifetimes *********************************
Function Calc3Lifetimes(a, tg, sigma, deltat, rho, theta1, theta2, bet, tau1, tau2, gamm, tau3, b, SyncPeriod, BinSize, Hist)
	Variable a, tg, sigma, deltat, rho, theta1, theta2, bet, tau1, tau2, gamm, tau3, b, SyncPeriod, BinSize
	Wave Hist
	Variable te = deltat + tg
	Duplicate/O Hist, Hist1, Hist2, Hist3, step
	step = (Sign(x-te)+1)/2
	Variable theta12=theta1/theta2
	Variable thinv=1/theta1+1/theta2
	
	Variable Q = Sqrt(2*Pi) * sigma + rho * theta1 * (1 + theta12)^(1/theta12)
	Variable P = (1+1/theta12) * (1+theta12)^(1/theta12)
	
	// tau1 contribution
	Variable phi = 1-bet-gamm
	Variable tau = tau1
	Hist1 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg)*tau )/Sqrt(2)/sigma/tau )* Exp(- x/tau)
	Hist1 += rho * step * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te))-1/(tau/theta1-1)*Exp(-(x-te)/theta1) )
	Hist1 *= phi
	
	// tau2 contribution
	phi = bet
	tau = tau2
	Hist2 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg)*tau )/Sqrt(2)/sigma/tau )* Exp(- x/tau)
	Hist2 += rho * step * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te))-1/(tau/theta1-1)*Exp(-(x-te)/theta1) )
	Hist2 *= phi
	
	// tau3 contribution
	phi = gamm
	tau = tau3
	Hist3 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg)*tau )/Sqrt(2)/sigma/tau )* Exp(- x/tau)
	Hist3 += rho * step * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te))-1/(tau/theta1-1)*Exp(-(x-te)/theta1) )
	Hist3 *= phi
	
	Hist = Hist1+Hist2+Hist3
	// add contribution from previous pulse
	// tau1 contribution
	phi = 1-bet-gamm
	tau = tau1
	Hist1 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg+SyncPeriod)*tau )/Sqrt(2)/sigma/tau )* Exp(- (x+SyncPeriod)/tau)
	Hist1 += rho * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te+SyncPeriod)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te+SyncPeriod))-1/(tau/theta1-1)*Exp(-(x-te+SyncPeriod)/theta1) )
	Hist1 *= phi
	
	// tau2 contribution
	phi = bet
	tau = tau2
	Hist2 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg+SyncPeriod)*tau )/Sqrt(2)/sigma/tau )* Exp(- (x+SyncPeriod)/tau)
	Hist2 += rho * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te+SyncPeriod)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te+SyncPeriod))-1/(tau/theta1-1)*Exp(-(x-te+SyncPeriod)/theta1) )
	Hist2 *= phi
	
	// tau3contribution
	phi = gamm
	tau = tau3
	Hist3 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg+SyncPeriod)*tau )/Sqrt(2)/sigma/tau )* Exp(- (x+SyncPeriod)/tau)
	Hist3 += rho * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te+SyncPeriod)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te+SyncPeriod))-1/(tau/theta1-1)*Exp(-(x-te+SyncPeriod)/theta1) )
	Hist3 *= phi

	Hist += Hist1+Hist2+Hist3
	Hist *= a/Q
	Hist += b
	Hist *= BinSize
				
	KillWaves/Z step, Hist1, Hist2, Hist3
End

//*****************************************************************************************//
// ************************* User-defined fitting procedures *******************************
//*****************************************************************************************//

Function IRFHist(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ Variable tb = tg + deltat
	//CurveFitDialog/ Variable tmtb= x-tb
	//CurveFitDialog/ Variable theta12 = theta1/theta2
	//CurveFitDialog/ Variable Hist = a * Exp( -(x-tg)^2/2/sigma^2)
	//CurveFitDialog/ Hist += a*rho * (1+1/theta12) * (1+theta12)^(1/theta12)* (Sign(tmtb)+1)/2 * Exp(-tmtb/theta1) * (1- Exp(-tmtb/theta2) ) 
	//CurveFitDialog/ Hist /= Sqrt(2*Pi)*sigma + rho*theta1*(theta12+1)^(1/theta12)
	//CurveFitDialog/ Hist += b
	//CurveFitDialog/ f(x) = Hist
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 8
	//CurveFitDialog/ w[0] = a
	//CurveFitDialog/ w[1] = tg
	//CurveFitDialog/ w[2] = sigma
	//CurveFitDialog/ w[3] = deltat
	//CurveFitDialog/ w[4] = rho
	//CurveFitDialog/ w[5] = theta1
	//CurveFitDialog/ w[6] = theta2
	//CurveFitDialog/ w[7] = b

	Variable tb = w[1] + w[3]
	Variable tmtb= x-tb
	Variable theta12 = w[5]/w[6]
	Variable Hist = w[0] * Exp( -(x-w[1])^2/2/w[2]^2)
	Hist += w[0]*w[4] * (1+1/theta12) * (1+theta12)^(1/theta12)* (Sign(tmtb)+1)/2 * Exp(-tmtb/w[5]) * (1- Exp(-tmtb/w[6]) ) 
	Hist /= Sqrt(2*Pi)*w[2] + w[4]*w[5]*(theta12+1)^(1/theta12)
	Hist += w[7]
	return Hist
End

Function Three_Lifetimes(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ Variable te = deltat + tg
	//CurveFitDialog/ Variable theta12=theta1/theta2
	//CurveFitDialog/ Variable thinv=1/theta1+1/theta2
	//CurveFitDialog/ Variable Q = Sqrt(2*Pi) * sigma + rho * theta1 * (1 + theta12)^(1/theta12)
	//CurveFitDialog/ Variable P = (1+1/theta12) * (1+theta12)^(1/theta12)
	//CurveFitDialog/ Variable step = (Sign(x-te)+1)/2
	//CurveFitDialog/ 
	//CurveFitDialog/ Variable phi = 1-bet
	//CurveFitDialog/ Variable tau = tau1
	//CurveFitDialog/ Variable Hist1 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg)*tau )/Sqrt(2)/sigma/tau )* Exp(- x/tau)
	//CurveFitDialog/ Hist1 += rho * step * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te))-1/(tau/theta1-1)*Exp(-(x-te)/theta1) )
	//CurveFitDialog/ Hist1 *= phi
	//CurveFitDialog/ 
	//CurveFitDialog/ phi = bet
	//CurveFitDialog/ tau = tau2
	//CurveFitDialog/ Variable Hist2 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg)*tau )/Sqrt(2)/sigma/tau )* Exp(- x/tau)
	//CurveFitDialog/ Hist2 += rho * step * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te))-1/(tau/theta1-1)*Exp(-(x-te)/theta1) )
	//CurveFitDialog/ Hist2 *= phi
	//CurveFitDialog/ 
	//CurveFitDialog/ Variable Hist = Hist1+Hist2
	//CurveFitDialog/ 
	//CurveFitDialog/ phi = 1-bet
	//CurveFitDialog/ tau = tau1
	//CurveFitDialog/ Hist1 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg+SyncPeriod)*tau )/Sqrt(2)/sigma/tau )* Exp(- (x+SyncPeriod)/tau)
	//CurveFitDialog/ Hist1 += rho * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te+SyncPeriod)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te+SyncPeriod))-1/(tau/theta1-1)*Exp(-(x-te+SyncPeriod)/theta1) )
	//CurveFitDialog/ Hist1 *= phi
	//CurveFitDialog/ 
	//CurveFitDialog/ phi = bet
	//CurveFitDialog/ tau = tau2
	//CurveFitDialog/ Hist2 = sigma/tau *  Sqrt(Pi/2) * Exp((sigma^2+2*tg*tau)/2/tau^2) * Erfc( (sigma^2 - (x-tg+SyncPeriod)*tau )/Sqrt(2)/sigma/tau )* Exp(- (x+SyncPeriod)/tau)
	//CurveFitDialog/ Hist2 += rho * P * (1/((tau*thinv-1)*(tau/theta1-1))*tau/theta2*Exp(-(x-te+SyncPeriod)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te+SyncPeriod))-1/(tau/theta1-1)*Exp(-(x-te+SyncPeriod)/theta1) )
	//CurveFitDialog/ Hist2 *= phi
	//CurveFitDialog/ 
	//CurveFitDialog/ Hist += Hist1+Hist2
	//CurveFitDialog/ Hist *= a/Q
	//CurveFitDialog/ Hist += b
	//CurveFitDialog/ f(x) = Hist
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 12
	//CurveFitDialog/ w[0] = a
	//CurveFitDialog/ w[1] = tg
	//CurveFitDialog/ w[2] = sigma
	//CurveFitDialog/ w[3] = deltat
	//CurveFitDialog/ w[4] = rho
	//CurveFitDialog/ w[5] = theta1
	//CurveFitDialog/ w[6] = theta2
	//CurveFitDialog/ w[7] = b
	//CurveFitDialog/ w[8] = tau1
	//CurveFitDialog/ w[9] = tau2
	//CurveFitDialog/ w[10] = bet
	//CurveFitDialog/ w[11] = tau3
	//CurveFitDialog/ w[12] = gamm
	//CurveFitDialog/ w[13] = SyncPeriod

	Variable te = w[3] + w[1]
	Variable theta12=w[5]/w[6]
	Variable thinv=1/w[5]+1/w[6]
	Variable Q = Sqrt(2*Pi) * w[2] + w[4] * w[5] * (1 + theta12)^(1/theta12)
	Variable P = (1+1/theta12) * (1+theta12)^(1/theta12)
	Variable step = (Sign(x-te)+1)/2
	
	Variable phi = 1-w[10]-w[12]
	Variable tau = w[8]
	Variable Hist1 = w[2]/tau *  Sqrt(Pi/2) * Exp((w[2]^2+2*w[1]*tau)/2/tau^2) * Erfc( (w[2]^2 - (x-w[1])*tau )/Sqrt(2)/w[2]/tau )* Exp(- x/tau)
	Hist1 += w[4] * step * P * (1/((tau*thinv-1)*(tau/w[5]-1))*tau/w[6]*Exp(-(x-te)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te))-1/(tau/w[5]-1)*Exp(-(x-te)/w[5]) )
	Hist1 = Hist1 * phi
	
	phi = w[10]
	tau = w[9]
	Variable Hist2 = w[2]/tau *  Sqrt(Pi/2) * Exp((w[2]^2+2*w[1]*tau)/2/tau^2) * Erfc( (w[2]^2 - (x-w[1])*tau )/Sqrt(2)/w[2]/tau )* Exp(- x/tau)
	Hist2 += w[4] * step * P * (1/((tau*thinv-1)*(tau/w[5]-1))*tau/w[6]*Exp(-(x-te)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te))-1/(tau/w[5]-1)*Exp(-(x-te)/w[5]) )
	Hist2 *= phi	
	
	phi = w[12]
	tau = w[11]
	Variable Hist3 = w[2]/tau *  Sqrt(Pi/2) * Exp((w[2]^2+2*w[1]*tau)/2/tau^2) * Erfc( (w[2]^2 - (x-w[1])*tau )/Sqrt(2)/w[2]/tau )* Exp(- x/tau)
	Hist3 += w[4] * step * P * (1/((tau*thinv-1)*(tau/w[5]-1))*tau/w[6]*Exp(-(x-te)/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te))-1/(tau/w[5]-1)*Exp(-(x-te)/w[5]) )
	Hist3 *= phi	
	
	Variable Hist = Hist1+Hist2+Hist3
	
	phi = 1-w[10]-w[12]
	tau = w[8]
	Hist1 = w[2]/tau *  Sqrt(Pi/2) * Exp((w[2]^2+2*w[1]*tau)/2/tau^2) * Erfc( (w[2]^2 - (x-w[1]+w[13])*tau )/Sqrt(2)/w[2]/tau )* Exp(- (x+w[13])/tau)
	Hist1 += w[4] * P * (1/((tau*thinv-1)*(tau/w[5]-1))*tau/w[6]*Exp(-(x-te+w[13])/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te+w[13]))-1/(tau/w[5]-1)*Exp(-(x-te+w[13])/w[5]) )
	Hist1 *= phi
	
	phi = w[10]
	tau = w[9]
	Hist2 = w[2]/tau *  Sqrt(Pi/2) * Exp((w[2]^2+2*w[1]*tau)/2/tau^2) * Erfc( (w[2]^2 - (x-w[1]+w[13])*tau )/Sqrt(2)/w[2]/tau )* Exp(- (x+w[13])/tau)
	Hist2 += w[4] * P * (1/((tau*thinv-1)*(tau/w[5]-1))*tau/w[6]*Exp(-(x-te+w[13])/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te+w[13]))-1/(tau/w[5]-1)*Exp(-(x-te+w[13])/w[5]) )
	Hist2 *= phi
	
	phi = w[12]
	tau = w[11]
	Hist3 = w[2]/tau *  Sqrt(Pi/2) * Exp((w[2]^2+2*w[1]*tau)/2/tau^2) * Erfc( (w[2]^2 - (x-w[1]+w[13])*tau )/Sqrt(2)/w[2]/tau )* Exp(- (x+w[13])/tau)
	Hist3 += w[4] * P * (1/((tau*thinv-1)*(tau/w[5]-1))*tau/w[6]*Exp(-(x-te+w[13])/tau)+1/(tau*thinv-1)*Exp(-thinv*(x-te+w[13]))-1/(tau/w[5]-1)*Exp(-(x-te+w[13])/w[5]) )
	Hist3 *= phi
	
	Hist += Hist1+Hist2+Hist3
	Hist *= w[0]/Q
	Hist += w[7]
	Hist *= 0.016
	return Hist
End


//*******************************************************************************************************
// Procedure to add a slider to a row of RGB images
//  Modified from LH procedure AxisSlider.ipf
// 25 FEB 09
// Changed the ValDisplay to SetVariable so that the control up and down arrows can be used to precisely move by one frame.

//*******************************************************************************************************
Function RGB3DImageSliderProc(name, value, event)
	String name			// name of this slider control
	Variable value		// value of slider
	Variable event		// bit field: bit 0: value set; 1: mouse down, //   2: mouse up, 3: mouse moved

	String dfSav= GetDataFolder(1)
	String grfName= WinName(0, 1)
	SetDataFolder root:Packages:RGB3DImageSlider:$(grfName)

	NVAR gLayer
	SVAR imageName//, fullimageName

	ModifyImage  $imageName plane=(gLayer)	
	SetDataFolder dfSav

	//  Tell us if there is an active LineProfile
	SVAR/Z imageGraphName=root:Packages:WMImProcess:LineProfile:imageGraphName
	if(SVAR_EXISTS(imageGraphName))
		if(cmpstr(imageGraphName,grfName)==0)
			ModifyGraph/W=$imageGraphName offset(lineProfileY)={0,0}			// This will fire the S_TraceOffsetInfo dependency
		endif
	endif	
		
	SVAR/Z imageGraphName=root:Packages:WMImProcess:ImageThreshold:ImGrfName
	if(SVAR_EXISTS(imageGraphName))
		if(cmpstr(imageGraphName,grfName)==0)
			WMImageThreshUpdate()
		endif
	endif
	
	return 0				// other return values reserved
End

//*******************************************************************************************************
Function RGBAppend3DImageSlider()
	String grfName= WinName(0, 1)
	DoWindow/F $grfName
	if( V_Flag==0 )
		return 0			// no top graph, exit
	endif

	String iName= WMTopImageGraph()		// find one top image in the top graph window
	if( strlen(iName) == 0 )
		DoAlert 0,"No image plot found"
		return 0
	endif
	
//	String/G fullimageName = WMGetImageWave(iName)	// get the wave associated with the top image.
//	Wave w= $fullimageName		
	Wave w= $WMGetImageWave(iName)	// get the wave associated with the top image.
	if(DimSize(w,3)<=0)
		DoAlert 0,"Need a 4D image"
		return 0
	endif
	
	ControlInfo WM3DAxis
	if( V_Flag != 0 )
		return 0			// already installed, do nothing
	endif
	
	String dfSav= GetDataFolder(1)
	NewDataFolder/S/O root:Packages
	NewDataFolder/S/O RGB3DImageSlider
	NewDataFolder/S/O $grfName
	
	Variable/G gLeftLim=0, gRightLim=DimSize(w,3)-1, gLayer=0	
	// check how many RGB frames there are (...T0..., ...T1..., ...T2..., ...)
//	if (strsearch(fullimageName, "T0", 0) < 0)
//		DoAlert 0, "String not found"
//		return 0
//	endif
//	do
//		gRightLim += 1	
//		String replace = "T" + num2str(gRightLim)
//		String wend = ReplaceString ("T0", fullimageName, replace)
//	while (WaveExists($(wend)))
//	gRightLim -= 1
	
	String/G imageName=nameOfWave(w)
	ControlInfo kwControlBar
	Variable/G gOriginalHeight= V_Height		// we append below original controls (if any)
	ControlBar gOriginalHeight+30

	GetWindow kwTopWin,gsize
	Variable kImageSliderLMargin= 80
	Slider RGB3DAxis,pos={V_left+10,gOriginalHeight+9},size={V_right-V_left-kImageSliderLMargin,16},proc=RGB3DImageSliderProc
	// uncomment the following line if you want do disable live updates when the slider moves.
	// Slider WM3DAxis live=0	
	Slider RGB3DAxis,limits={0,gRightLim,1},value= 0,vert= 0,ticks=0,side=0,variable=gLayer	
	
	SetVariable RGB3DVal,pos={V_right-kImageSliderLMargin+15,gOriginalHeight+9},size={60,14}
	SetVariable RGB3DVal,limits={0,INF,1},title=" ",proc=RGB3DImageSliderSetVarProc
	
	String cmd
	sprintf cmd,"SetVariable RGB3DVal,value=%s",GetDataFolder(1)+"gLayer"
	Execute cmd

	ModifyImage $imageName plane=0
	WaveStats/Q w
	ModifyImage $imageName ctab= {V_min,V_max,,0}	// missing ctb to leave it unchanced.
	
	SetDataFolder dfSav
End


//*******************************************************************************************************
Function RGB3DImageSliderSetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
		// comment the following line if you want to disable live updates.
		case 3: // Live update
			Variable dval = sva.dval
			RGB3DImageSliderProc("",0,0)
			break
	endswitch

	return 0
End

